#ifdef HAVE_CONFIG_H
# include <config.h>
#endif	

#line 5 "grecs-lex.c"

#define  YY_INT_ALIGNED short int

/* A lexical scanner generated by flex */

/* %not-for-header */
/* %if-c-only */
/* %if-not-reentrant */

/* %endif */
/* %endif */
/* %ok-for-header */

#define FLEX_SCANNER
#define YY_FLEX_MAJOR_VERSION 2
#define YY_FLEX_MINOR_VERSION 6
#define YY_FLEX_SUBMINOR_VERSION 4
#if YY_FLEX_SUBMINOR_VERSION > 0
#define FLEX_BETA
#endif

/* %if-c++-only */
/* %endif */

/* %if-c-only */

/* %endif */

/* %if-c-only */

/* %endif */

/* First, we deal with  platform-specific or compiler-specific issues. */

/* begin standard C headers. */
/* %if-c-only */
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>
/* %endif */

/* %if-tables-serialization */
/* %endif */
/* end standard C headers. */

/* %if-c-or-c++ */
/* flex integer type definitions */

#ifndef FLEXINT_H
#define FLEXINT_H

/* C99 systems have <inttypes.h>. Non-C99 systems may or may not. */

#if defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L

/* C99 says to define __STDC_LIMIT_MACROS before including stdint.h,
 * if you want the limit (max/min) macros for int types. 
 */
#ifndef __STDC_LIMIT_MACROS
#define __STDC_LIMIT_MACROS 1
#endif

#include <inttypes.h>
typedef int8_t flex_int8_t;
typedef uint8_t flex_uint8_t;
typedef int16_t flex_int16_t;
typedef uint16_t flex_uint16_t;
typedef int32_t flex_int32_t;
typedef uint32_t flex_uint32_t;
#else
typedef signed char flex_int8_t;
typedef short int flex_int16_t;
typedef int flex_int32_t;
typedef unsigned char flex_uint8_t; 
typedef unsigned short int flex_uint16_t;
typedef unsigned int flex_uint32_t;

/* Limits of integral types. */
#ifndef INT8_MIN
#define INT8_MIN               (-128)
#endif
#ifndef INT16_MIN
#define INT16_MIN              (-32767-1)
#endif
#ifndef INT32_MIN
#define INT32_MIN              (-2147483647-1)
#endif
#ifndef INT8_MAX
#define INT8_MAX               (127)
#endif
#ifndef INT16_MAX
#define INT16_MAX              (32767)
#endif
#ifndef INT32_MAX
#define INT32_MAX              (2147483647)
#endif
#ifndef UINT8_MAX
#define UINT8_MAX              (255U)
#endif
#ifndef UINT16_MAX
#define UINT16_MAX             (65535U)
#endif
#ifndef UINT32_MAX
#define UINT32_MAX             (4294967295U)
#endif

#ifndef SIZE_MAX
#define SIZE_MAX               (~(size_t)0)
#endif

#endif /* ! C99 */

#endif /* ! FLEXINT_H */

/* %endif */

/* begin standard C++ headers. */
/* %if-c++-only */
/* %endif */

/* TODO: this is always defined, so inline it */
#define grecs_grecs_const const

#if defined(__GNUC__) && __GNUC__ >= 3
#define grecs_grecs_noreturn __attribute__((__noreturn__))
#else
#define grecs_grecs_noreturn
#endif

/* %not-for-header */
/* Returned upon end-of-file. */
#define YY_NULL 0
/* %ok-for-header */

/* %not-for-header */
/* Promotes a possibly negative, possibly signed char to an
 *   integer in range [0..255] for use as an array index.
 */
#define YY_SC_TO_UI(c) ((YY_CHAR) (c))
/* %ok-for-header */

/* %if-reentrant */
/* %endif */

/* %if-not-reentrant */

/* %endif */

/* Enter a start condition.  This macro really ought to take a parameter,
 * but we do it the disgusting crufty way forced on us by the ()-less
 * definition of BEGIN.
 */
#define BEGIN (grecs_grecs__start) = 1 + 2 *
/* Translate the current start state into a value that can be later handed
 * to BEGIN to return to the state.  The YYSTATE alias is for lex
 * compatibility.
 */
#define YY_START (((grecs_grecs__start) - 1) / 2)
#define YYSTATE YY_START
/* Action number for EOF rule of a given start state. */
#define YY_STATE_EOF(state) (YY_END_OF_BUFFER + state + 1)
/* Special action meaning "start processing a new file". */
#define YY_NEW_FILE grecs_grecs_restart( grecs_grecs_in  )
#define YY_END_OF_BUFFER_CHAR 0

/* Size of default input buffer. */
#ifndef YY_BUF_SIZE
#ifdef __ia64__
/* On IA-64, the buffer size is 16k, not 8k.
 * Moreover, YY_BUF_SIZE is 2*YY_READ_BUF_SIZE in the general case.
 * Ditto for the __ia64__ case accordingly.
 */
#define YY_BUF_SIZE 32768
#else
#define YY_BUF_SIZE 16384
#endif /* __ia64__ */
#endif

/* The state buf must be large enough to hold one state per character in the main buffer.
 */
#define YY_STATE_BUF_SIZE   ((YY_BUF_SIZE + 2) * sizeof(grecs_grecs__state_type))

#ifndef YY_TYPEDEF_YY_BUFFER_STATE
#define YY_TYPEDEF_YY_BUFFER_STATE
typedef struct grecs_grecs__buffer_state *YY_BUFFER_STATE;
#endif

#ifndef YY_TYPEDEF_YY_SIZE_T
#define YY_TYPEDEF_YY_SIZE_T
typedef size_t grecs_grecs__size_t;
#endif

/* %if-not-reentrant */
extern int grecs_grecs_leng;
/* %endif */

/* %if-c-only */
/* %if-not-reentrant */
extern FILE *grecs_grecs_in, *grecs_grecs_out;
/* %endif */
/* %endif */

#define EOB_ACT_CONTINUE_SCAN 0
#define EOB_ACT_END_OF_FILE 1
#define EOB_ACT_LAST_MATCH 2
    
    #define YY_LESS_LINENO(n)
    #define YY_LINENO_REWIND_TO(ptr)
    
/* Return all but the first "n" matched characters back to the input stream. */
#define grecs_grecs_less(n) \
	do \
		{ \
		/* Undo effects of setting up grecs_grecs_text. */ \
        int grecs_grecs_less_macro_arg = (n); \
        YY_LESS_LINENO(grecs_grecs_less_macro_arg);\
		*grecs_grecs__cp = (grecs_grecs__hold_char); \
		YY_RESTORE_YY_MORE_OFFSET \
		(grecs_grecs__c_buf_p) = grecs_grecs__cp = grecs_grecs__bp + grecs_grecs_less_macro_arg - YY_MORE_ADJ; \
		YY_DO_BEFORE_ACTION; /* set up grecs_grecs_text again */ \
		} \
	while ( 0 )
#define unput(c) grecs_grecs_unput( c, (grecs_grecs_text_ptr)  )

#ifndef YY_STRUCT_YY_BUFFER_STATE
#define YY_STRUCT_YY_BUFFER_STATE
struct grecs_grecs__buffer_state
	{
/* %if-c-only */
	FILE *grecs_grecs__input_file;
/* %endif */

/* %if-c++-only */
/* %endif */

	char *grecs_grecs__ch_buf;		/* input buffer */
	char *grecs_grecs__buf_pos;		/* current position in input buffer */

	/* Size of input buffer in bytes, not including room for EOB
	 * characters.
	 */
	int grecs_grecs__buf_size;

	/* Number of characters read into grecs_grecs__ch_buf, not including EOB
	 * characters.
	 */
	int grecs_grecs__n_chars;

	/* Whether we "own" the buffer - i.e., we know we created it,
	 * and can realloc() it to grow it, and should free() it to
	 * delete it.
	 */
	int grecs_grecs__is_our_buffer;

	/* Whether this is an "interactive" input source; if so, and
	 * if we're using stdio for input, then we want to use getc()
	 * instead of fread(), to make sure we stop fetching input after
	 * each newline.
	 */
	int grecs_grecs__is_interactive;

	/* Whether we're considered to be at the beginning of a line.
	 * If so, '^' rules will be active on the next match, otherwise
	 * not.
	 */
	int grecs_grecs__at_bol;

    int grecs_grecs__bs_lineno; /**< The line count. */
    int grecs_grecs__bs_column; /**< The column count. */

	/* Whether to try to fill the input buffer when we reach the
	 * end of it.
	 */
	int grecs_grecs__fill_buffer;

	int grecs_grecs__buffer_status;

#define YY_BUFFER_NEW 0
#define YY_BUFFER_NORMAL 1
	/* When an EOF's been seen but there's still some text to process
	 * then we mark the buffer as YY_EOF_PENDING, to indicate that we
	 * shouldn't try reading from the input source any more.  We might
	 * still have a bunch of tokens to match, though, because of
	 * possible backing-up.
	 *
	 * When we actually see the EOF, we change the status to "new"
	 * (via grecs_grecs_restart()), so that the user can continue scanning by
	 * just pointing grecs_grecs_in at a new input file.
	 */
#define YY_BUFFER_EOF_PENDING 2

	};
#endif /* !YY_STRUCT_YY_BUFFER_STATE */

/* %if-c-only Standard (non-C++) definition */
/* %not-for-header */
/* %if-not-reentrant */

/* Stack of input buffers. */
static size_t grecs_grecs__buffer_stack_top = 0; /**< index of top of stack. */
static size_t grecs_grecs__buffer_stack_max = 0; /**< capacity of stack. */
static YY_BUFFER_STATE * grecs_grecs__buffer_stack = NULL; /**< Stack as an array. */
/* %endif */
/* %ok-for-header */

/* %endif */

/* We provide macros for accessing buffer states in case in the
 * future we want to put the buffer states in a more general
 * "scanner state".
 *
 * Returns the top of the stack, or NULL.
 */
#define YY_CURRENT_BUFFER ( (grecs_grecs__buffer_stack) \
                          ? (grecs_grecs__buffer_stack)[(grecs_grecs__buffer_stack_top)] \
                          : NULL)
/* Same as previous macro, but useful when we know that the buffer stack is not
 * NULL or when we need an lvalue. For internal use only.
 */
#define YY_CURRENT_BUFFER_LVALUE (grecs_grecs__buffer_stack)[(grecs_grecs__buffer_stack_top)]

/* %if-c-only Standard (non-C++) definition */

/* %if-not-reentrant */
/* %not-for-header */
/* grecs_grecs__hold_char holds the character lost when grecs_grecs_text is formed. */
static char grecs_grecs__hold_char;
static int grecs_grecs__n_chars;		/* number of characters read into grecs_grecs__ch_buf */
int grecs_grecs_leng;

/* Points to current character in buffer. */
static char *grecs_grecs__c_buf_p = NULL;
static int grecs_grecs__init = 0;		/* whether we need to initialize */
static int grecs_grecs__start = 0;	/* start state number */

/* Flag which is used to allow grecs_grecs_wrap()'s to do buffer switches
 * instead of setting up a fresh grecs_grecs_in.  A bit of a hack ...
 */
static int grecs_grecs__did_buffer_switch_on_eof;
/* %ok-for-header */

/* %endif */

void grecs_grecs_restart ( FILE *input_file  );
void grecs_grecs__switch_to_buffer ( YY_BUFFER_STATE new_buffer  );
YY_BUFFER_STATE grecs_grecs__create_buffer ( FILE *file, int size  );
void grecs_grecs__delete_buffer ( YY_BUFFER_STATE b  );
void grecs_grecs__flush_buffer ( YY_BUFFER_STATE b  );
void grecs_grecs_push_buffer_state ( YY_BUFFER_STATE new_buffer  );
void grecs_grecs_pop_buffer_state ( void );

static void grecs_grecs_ensure_buffer_stack ( void );
static void grecs_grecs__load_buffer_state ( void );
static void grecs_grecs__init_buffer ( YY_BUFFER_STATE b, FILE *file  );
#define YY_FLUSH_BUFFER grecs_grecs__flush_buffer( YY_CURRENT_BUFFER )

YY_BUFFER_STATE grecs_grecs__scan_buffer ( char *base, grecs_grecs__size_t size  );
YY_BUFFER_STATE grecs_grecs__scan_string ( const char *grecs_grecs__str  );
YY_BUFFER_STATE grecs_grecs__scan_bytes ( const char *bytes, int len  );

/* %endif */

void *grecs_grecs_alloc ( grecs_grecs__size_t  );
void *grecs_grecs_realloc ( void *, grecs_grecs__size_t  );
void grecs_grecs_free ( void *  );

#define grecs_grecs__new_buffer grecs_grecs__create_buffer
#define grecs_grecs__set_interactive(is_interactive) \
	{ \
	if ( ! YY_CURRENT_BUFFER ){ \
        grecs_grecs_ensure_buffer_stack (); \
		YY_CURRENT_BUFFER_LVALUE =    \
            grecs_grecs__create_buffer( grecs_grecs_in, YY_BUF_SIZE ); \
	} \
	YY_CURRENT_BUFFER_LVALUE->grecs_grecs__is_interactive = is_interactive; \
	}
#define grecs_grecs__set_bol(at_bol) \
	{ \
	if ( ! YY_CURRENT_BUFFER ){\
        grecs_grecs_ensure_buffer_stack (); \
		YY_CURRENT_BUFFER_LVALUE =    \
            grecs_grecs__create_buffer( grecs_grecs_in, YY_BUF_SIZE ); \
	} \
	YY_CURRENT_BUFFER_LVALUE->grecs_grecs__at_bol = at_bol; \
	}
#define YY_AT_BOL() (YY_CURRENT_BUFFER_LVALUE->grecs_grecs__at_bol)

/* %% [1.0] grecs_grecs_text/grecs_grecs_in/grecs_grecs_out/grecs_grecs__state_type/grecs_grecs_lineno etc. def's & init go here */
/* Begin user sect3 */

#define FLEX_DEBUG
typedef flex_uint8_t YY_CHAR;

FILE *grecs_grecs_in = NULL, *grecs_grecs_out = NULL;

typedef int grecs_grecs__state_type;

extern int grecs_grecs_lineno;
int grecs_grecs_lineno = 1;

extern char *grecs_grecs_text;
#ifdef grecs_grecs_text_ptr
#undef grecs_grecs_text_ptr
#endif
#define grecs_grecs_text_ptr grecs_grecs_text

/* %% [1.5] DFA */

/* %if-c-only Standard (non-C++) definition */

static grecs_grecs__state_type grecs_grecs__get_previous_state ( void );
static grecs_grecs__state_type grecs_grecs__try_NUL_trans ( grecs_grecs__state_type current_state  );
static int grecs_grecs__get_next_buffer ( void );
static void grecs_grecs_noreturn grecs_grecs__fatal_error ( const char* msg  );

/* %endif */

/* Done after the current pattern has been matched and before the
 * corresponding action - sets up grecs_grecs_text.
 */
#define YY_DO_BEFORE_ACTION \
	(grecs_grecs_text_ptr) = grecs_grecs__bp; \
/* %% [2.0] code to fiddle grecs_grecs_text and grecs_grecs_leng for grecs_grecs_more() goes here \ */\
	grecs_grecs_leng = (int) (grecs_grecs__cp - grecs_grecs__bp); \
	(grecs_grecs__hold_char) = *grecs_grecs__cp; \
	*grecs_grecs__cp = '\0'; \
/* %% [3.0] code to copy grecs_grecs_text_ptr to grecs_grecs_text[] goes here, if %array \ */\
	(grecs_grecs__c_buf_p) = grecs_grecs__cp;
/* %% [4.0] data tables for the DFA and the user's section 1 definitions go here */
#define YY_NUM_RULES 41
#define YY_END_OF_BUFFER 42
/* This struct is not used in this scanner,
   but its presence is necessary. */
struct grecs_grecs__trans_info
	{
	flex_int32_t grecs_grecs__verify;
	flex_int32_t grecs_grecs__nxt;
	};
static const flex_int16_t grecs_grecs__accept[262] =
    {   0,
        0,    0,    2,    2,    0,    0,    0,    0,    0,    0,
       42,   16,   37,   38,   40,   13,   40,   39,   17,   17,
       16,   37,   13,    2,    4,    3,   41,   36,   41,   41,
       41,   28,   41,   20,   22,   21,   19,   19,   16,   37,
        0,   23,    0,   13,   13,   12,   17,    1,   15,   16,
       37,    0,   13,   13,   13,   13,   13,   13,   13,    2,
        3,    3,    5,    0,   36,    0,    0,   28,    0,   27,
       26,   20,   19,   19,   18,   25,   24,   15,   14,    0,
       16,   16,   16,    0,    0,    0,    0,    0,    0,    0,
       13,   13,   13,   13,   13,   13,   13,   13,   13,   13,

       13,   13,   13,    0,    0,    0,    0,   31,   16,   16,
        0,    0,    0,    0,    0,    0,    0,    0,   13,   13,
       13,   13,   13,   13,   13,   13,   13,   13,   13,   13,
        0,    0,    0,    0,    0,    0,   16,    0,   29,    0,
        0,    0,    0,    0,    0,    0,   13,   13,   13,   13,
       13,   13,   13,   13,   13,   13,   13,   13,    0,    0,
       34,    0,    0,    0,    0,   30,    0,    0,    0,    0,
        0,    0,    0,   13,   13,   13,   13,   13,   13,    6,
       13,   13,   13,   13,   13,   13,    0,    0,   32,    0,
        0,    6,    0,    0,    0,    0,    0,    0,   13,   13,

       13,   13,   13,   13,   13,   13,    7,   13,    0,   35,
        0,   33,    0,    0,    0,    0,    7,    0,   13,   13,
       13,   11,   13,    9,   13,   13,    0,   11,    0,    9,
        0,    0,   13,   13,   13,   13,    8,   13,   13,    0,
        8,    0,    0,   13,   13,   13,   10,    0,    0,   10,
       13,   13,    0,   13,   13,    0,   13,   13,    0,   13,
        0
    } ;

static const YY_CHAR grecs_grecs__ec[256] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    2,    3,
        1,    4,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    5,    1,    6,    7,    1,    1,    1,    8,    9,
        9,   10,    1,    9,   11,    1,   12,   13,   14,   14,
       14,   14,   14,   14,   14,   14,   14,    1,    9,   15,
        1,    1,    1,    1,   16,   16,   16,   16,   16,   16,
       16,   16,   16,   16,   16,   16,   16,   16,   16,   16,
       16,   16,   16,   16,   16,   16,   16,   16,   16,   16,
        1,   17,    1,    1,   18,    1,   19,   20,   21,   22,

       23,   16,   24,   16,   25,   16,   16,   26,   16,   27,
       28,   16,   16,   29,   16,   16,   30,   16,   31,   16,
       16,   16,    9,    1,    9,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,

        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1
    } ;

static const YY_CHAR grecs_grecs__meta[32] =
    {   0,
        1,    2,    3,    4,    2,    5,    6,    4,    4,    7,
        8,    9,    8,    8,    1,   10,   11,   10,   10,   10,
       10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
       10
    } ;

static const flex_int16_t grecs_grecs__base[307] =
    {   0,
        0,   14,   22,   24,  918,   28,   27,   33,   49,   60,
      920,    0,    9,  922,   31,   71,  922,  922,    0,   28,
      904,   80,   86,    0,  922,   31,  915,  922,   75,   39,
      922,  922,  914,   37,  922,  922,  922,   87,    0,  102,
       54,  922,  913,  116,  912,  922,    0,  922,  911,  147,
      118,  163,  193,  114,  893,  883,  884,  885,  890,    0,
       98,  103,  922,  905,  922,   99,   97,  922,  904,  922,
      922,  119,  922,  122,  922,  922,  922,  903,  922,    0,
      224,  241,  132,    0,  131,  885,  875,  876,  877,  882,
      257,  123,   64,  143,  147,  137,  173,  178,  877,  870,

      877,  870,  867,  154,  889,  146,  270,  922,  281,  293,
      698,  195,  197,  659,  651,  658,  625,  622,  309,  148,
      156,   25,  163,  164,  181,  623,  621,  620,  621,  616,
      200,  322,  631,  333,  198,  632,  344,  631,  922,  625,
      620,  596,  594,  595,  557,  552,  205,  206,  202,  203,
      218,  211,  230,  556,  548,  526,  245,  509,  217,    0,
      922,  524,  511,  355,  515,  922,  510,  246,  492,  484,
      482,  249,  486,  152,  237,  258,  262,  254,  275,  922,
      296,  299,  301,  468,  482,  456,  478,  477,  922,  476,
      310,  922,  312,  315,  326,  456,  417,  392,  339,  361,

      252,  265,  415,  414,  393,  412,  922,  390,  410,  922,
      409,  922,  408,  407,  386,  405,  922,  383,  316,  273,
      403,  922,  401,  922,  366,  328,  400,  922,  399,  922,
      362,  377,  370,  384,  388,  352,  922,  315,  318,  306,
      922,  304,  277,  323,  242,  260,  922,  334,  224,  922,
      356,  192,  374,  378,  150,  382,  398,    0,  166,   83,
      922,  421,  432,  443,  454,  465,  476,  487,  498,  509,
      520,  531,  542,  553,  564,  575,  586,  597,  608,  619,
       32,  630,  637,  643,  654,  665,  675,  685,  696,  707,
      718,  729,  740,  751,  762,  773,  784,  795,  806,  817,

      828,  839,  850,  861,  872,  883
    } ;

static const flex_int16_t grecs_grecs__def[307] =
    {   0,
      262,  262,  263,  263,  264,  264,  265,  265,  266,  266,
      261,  267,  261,  261,  268,  269,  261,  261,  270,  270,
      267,  261,  269,  271,  261,  272,  273,  261,  273,  274,
      261,  261,  275,  276,  261,  261,  261,  261,  267,  261,
      268,  261,  277,   23,  278,  261,  270,  261,  279,  280,
      261,  261,  278,   44,   44,   44,   44,   44,   44,  271,
      272,  272,  261,  273,  261,  273,  274,  261,  275,  261,
      261,  276,  261,  261,  261,  261,  261,  279,  261,  281,
      280,  282,   81,   52,  261,  261,  261,  261,  261,  261,
      278,  278,  278,  278,  278,  278,  278,   44,   44,   44,

       44,   44,   44,  273,  283,  284,  261,  261,  285,  282,
      261,  261,  261,  261,  261,  261,  261,  261,  278,  278,
      278,  278,  278,  278,  286,   44,   44,   44,   44,   44,
      273,  261,  283,  287,  284,  288,  285,  288,  261,  289,
      290,  261,  261,  261,  261,  261,  278,  278,  278,  278,
      278,  286,  291,   44,   44,   44,   44,   44,  273,  132,
      261,  292,  261,  287,  289,  261,  290,  293,  261,  261,
      261,  261,  261,  278,  278,  278,  278,  278,  291,  261,
      291,   44,   44,   44,  294,   44,  295,  292,  261,  296,
      293,  261,  293,  261,  261,  261,  297,  261,  278,  278,

      278,  278,  298,  299,   44,  294,  261,   44,  295,  261,
      296,  261,  300,  301,  261,  297,  261,  261,  278,  278,
      298,  261,  299,  261,  302,   44,  300,  261,  301,  261,
      303,  261,  304,  278,  302,  304,  261,  235,  305,  303,
      261,  303,  306,  304,  235,  305,  261,  303,  306,  261,
      304,  235,  303,  304,  235,  303,  304,  235,  303,  304,
        0,  261,  261,  261,  261,  261,  261,  261,  261,  261,
      261,  261,  261,  261,  261,  261,  261,  261,  261,  261,
      261,  261,  261,  261,  261,  261,  261,  261,  261,  261,
      261,  261,  261,  261,  261,  261,  261,  261,  261,  261,

      261,  261,  261,  261,  261,  261
    } ;

static const flex_int16_t grecs_grecs__nxt[954] =
    {   0,
      261,   13,   14,   13,   13,   15,   16,   17,   18,   19,
       40,   20,   40,   40,   21,   22,   14,   13,   22,   15,
       23,   17,   18,   19,   25,   20,   25,   46,   21,   31,
       28,   26,   32,   26,   29,   31,   42,   48,   32,   49,
       62,  105,   63,   33,   68,  149,   73,   43,   74,   33,
       35,   36,   35,   35,   35,   69,   35,   35,   37,   42,
       38,   35,   36,   35,   35,   35,   46,   35,   35,   37,
       43,   38,   45,   46,   45,   45,   45,   65,   45,   45,
       45,   51,   45,   40,   51,  237,   52,   53,   46,   45,
       53,   45,  121,   45,   45,   45,   75,   45,   75,   54,

       66,   65,   68,   40,   55,   40,   40,  261,   56,  261,
       57,   58,   62,   69,   63,   97,   59,   45,   97,   51,
       45,   40,   51,  104,   52,   46,   98,   98,   73,   44,
       74,   75,  112,   75,   44,  112,  261,  261,   44,   46,
       44,   44,  120,  113,  113,   46,   44,   39,   39,   46,
       46,   80,   80,   39,   46,  124,   65,   81,   46,   39,
       39,   39,  135,   83,   84,   46,   46,   84,  241,  122,
      147,  123,  258,  199,   97,   46,   85,   97,  125,   97,
      131,   86,   97,   46,  148,   87,  153,   88,   89,  150,
       98,   98,  151,   90,   53,   46,  112,   53,  112,  112,

      141,  112,   65,  261,   46,   46,   91,   46,   46,  113,
      113,   92,  255,   46,  261,   93,  153,   94,   95,   65,
       46,  187,  159,   96,   39,  177,  250,  176,  106,   80,
       39,  174,  180,  175,   39,  181,   39,   39,   39,   46,
       83,   39,  107,  108,  178,  107,  185,  109,  192,  185,
      197,  193,  111,  197,   46,   39,   46,   39,   97,   46,
       46,   97,  247,  185,   46,  200,  185,   46,  252,  119,
      119,  107,  108,  219,  107,   46,  136,  180,  202,  250,
      181,  111,  138,  139,  138,  138,  138,  201,  138,  138,
      138,  220,  138,   39,  107,  108,  234,  107,  180,  109,

      203,  181,  204,  203,  111,  204,  241,   39,  241,   39,
       97,   46,  192,   97,  192,  193,  213,  193,   46,  213,
      247,  119,  119,  160,  161,  237,  160,  214,  162,  239,
      214,  248,  239,  163,  107,  108,  241,  107,  233,  136,
      203,   46,  245,  203,  111,  138,  139,  138,  138,  138,
      251,  138,  138,  138,  237,  138,  107,  108,  237,  107,
      253,  136,  204,   46,  241,  204,  111,  236,  237,  236,
      236,  236,  237,  236,  236,  236,  241,  236,  243,  242,
      237,  243,  254,  238,  241,  239,   46,  244,  239,  236,
      237,  236,  236,  236,  256,  236,  236,  236,  257,  236,

      237,  230,  228,  224,  259,  222,  232,  217,  231,  230,
      228,  212,  210,  226,  207,  225,  224,  222,  218,  217,
      260,   12,   12,   12,   12,   12,   12,   12,   12,   12,
       12,   12,   24,   24,   24,   24,   24,   24,   24,   24,
       24,   24,   24,   27,   27,   27,   27,   27,   27,   27,
       27,   27,   27,   27,   30,   30,   30,   30,   30,   30,
       30,   30,   30,   30,   30,   34,   34,   34,   34,   34,
       34,   34,   34,   34,   34,   34,   39,  215,  212,  189,
      210,   39,  208,   39,  207,   39,   39,   41,   41,  205,
       41,   41,   41,   41,   41,   41,   41,   41,   44,   44,

       44,   44,   44,   44,   44,   44,   44,   44,   44,   47,
      198,  196,  195,  194,   47,  168,   47,  166,   47,   47,
       60,   60,  190,   60,   60,   60,  189,   60,   60,   60,
       60,   61,   61,  186,   61,   61,   61,   61,   61,   61,
       61,   61,   64,   64,   64,   64,   64,   64,   64,   64,
       64,   64,   64,   67,   67,  184,   67,   67,   67,   67,
       67,   67,   67,   67,   70,   70,   70,   70,   70,   70,
       70,   70,   70,   70,   70,   72,  183,  182,  173,  172,
       72,   72,   72,   72,   72,   72,   76,   76,   76,   76,
       76,   76,   76,   76,   76,   76,   76,   45,   45,   45,

       45,   45,   45,   45,   45,   45,   45,   45,   78,   78,
       78,   78,   78,   78,   78,   78,   78,   78,   78,   82,
      171,  170,  169,   82,   82,  168,   82,  166,   82,   82,
      110,  110,  110,  139,  139,  110,  132,  110,  110,  110,
      110,  133,  158,  157,  133,  156,  133,  134,  155,  154,
      146,  145,  134,  134,  137,  137,  137,  137,  137,  137,
      137,  137,  137,  137,  137,  152,  152,  152,  152,  152,
      152,  152,  152,  152,  152,  152,  164,  164,  144,  143,
      164,  142,  164,  164,  164,  138,  138,  138,  138,  138,
      138,  138,  138,  138,  138,  138,  165,  165,  165,  165,

      165,  165,  165,  165,  165,  165,  165,  167,  167,  140,
      167,  167,  167,  167,  167,  167,  167,  167,  179,  179,
      179,  179,  179,  179,  179,  179,  179,  179,  179,  188,
      188,  188,  188,  188,  188,  188,  188,  188,  188,  188,
      191,  191,  191,  191,  191,  191,  191,  191,  191,  191,
      191,  206,  206,  206,  206,  206,  206,  206,  206,  206,
      206,  206,  209,  209,  209,  209,  209,  209,  209,  209,
      209,  209,  209,  211,  211,  211,  211,  211,  211,  211,
      211,  211,  211,  211,  216,  216,  216,  216,  216,  216,
      216,  216,  216,  216,  216,  221,  221,  221,  221,  221,

      221,  221,  221,  221,  221,  221,  223,  223,  223,  223,
      223,  223,  223,  223,  223,  223,  223,  227,  227,  227,
      227,  227,  227,  227,  227,  227,  227,  227,  229,  229,
      229,  229,  229,  229,  229,  229,  229,  229,  229,  235,
      235,  235,  235,  235,  235,  235,  235,  235,  235,  235,
      240,  240,  240,  240,  240,  240,  240,  240,  240,  240,
      240,  236,  236,  236,  236,  236,  236,  236,  236,  236,
      236,  236,  246,  246,  246,  246,  246,  246,  246,  246,
      246,  246,  246,  249,  249,  249,  249,  249,  249,  249,
      249,  249,  249,  249,  132,  130,  129,  128,  127,  126,

      118,  117,  116,  115,  114,   79,   71,   65,  103,  102,
      101,  100,   99,   79,   46,   77,   71,   65,   50,  261,
       28,   11,  261,  261,  261,  261,  261,  261,  261,  261,
      261,  261,  261,  261,  261,  261,  261,  261,  261,  261,
      261,  261,  261,  261,  261,  261,  261,  261,  261,  261,
      261,  261,  261
    } ;

static const flex_int16_t grecs_grecs__chk[954] =
    {   0,
        0,    1,    1,    1,    1,    1,    1,    1,    1,    1,
       13,    1,   13,   13,    1,    2,    2,    2,    2,    2,
        2,    2,    2,    2,    3,    2,    4,  122,    2,    7,
        6,    3,    7,    4,    6,    8,   15,   20,    8,   20,
       26,  281,   26,    7,   30,  122,   34,   15,   34,    8,
        9,    9,    9,    9,    9,   30,    9,    9,    9,   41,
        9,   10,   10,   10,   10,   10,   93,   10,   10,   10,
       41,   10,   16,   16,   16,   16,   16,   29,   16,   16,
       16,   22,   16,   22,   22,  260,   22,   23,   23,   23,
       23,   23,   93,   23,   23,   23,   38,   23,   38,   23,

       29,   66,   67,   40,   23,   40,   40,   61,   23,   61,
       23,   23,   62,   67,   62,   54,   23,   44,   54,   51,
       44,   51,   51,   66,   51,   92,   54,   54,   72,   44,
       72,   74,   85,   74,   44,   85,   83,   83,   44,   96,
       44,   44,   92,   85,   85,   94,   44,   50,   83,   95,
      120,  106,   50,   50,  174,   96,  104,   50,  121,   50,
       50,   50,  106,   50,   52,  123,  124,   52,  259,   94,
      120,   95,  255,  174,   97,   97,   52,   97,   97,   98,
      104,   52,   98,  125,  121,   52,  125,   52,   52,  123,
       98,   98,  124,   52,   53,   53,  112,   53,  113,  112,

      112,  113,  131,  135,  149,  150,   53,  147,  148,  113,
      113,   53,  252,  152,  135,   53,  152,   53,   53,  159,
      151,  159,  131,   53,   81,  150,  249,  149,   81,   81,
       81,  147,  153,  148,   81,  153,   81,   81,   81,  175,
       81,   82,   82,   82,  151,   82,  157,   82,  168,  157,
      172,  168,   82,  172,  201,   82,  178,   82,   91,   91,
      176,   91,  246,  177,  177,  175,  177,  202,  245,   91,
       91,  107,  107,  201,  107,  220,  107,  179,  178,  243,
      179,  107,  109,  109,  109,  109,  109,  176,  109,  109,
      109,  202,  109,  110,  110,  110,  220,  110,  181,  110,

      182,  181,  183,  182,  110,  183,  242,  110,  240,  110,
      119,  119,  191,  119,  193,  191,  194,  193,  219,  194,
      239,  119,  119,  132,  132,  244,  132,  195,  132,  226,
      195,  242,  226,  132,  134,  134,  248,  134,  219,  134,
      199,  199,  238,  199,  134,  137,  137,  137,  137,  137,
      244,  137,  137,  137,  236,  137,  164,  164,  251,  164,
      248,  164,  200,  200,  231,  200,  164,  225,  225,  225,
      225,  225,  233,  225,  225,  225,  253,  225,  232,  231,
      254,  232,  251,  225,  256,  234,  234,  233,  234,  235,
      235,  235,  235,  235,  253,  235,  235,  235,  254,  235,

      257,  229,  227,  223,  256,  221,  218,  216,  215,  214,
      213,  211,  209,  208,  206,  205,  204,  203,  198,  197,
      257,  262,  262,  262,  262,  262,  262,  262,  262,  262,
      262,  262,  263,  263,  263,  263,  263,  263,  263,  263,
      263,  263,  263,  264,  264,  264,  264,  264,  264,  264,
      264,  264,  264,  264,  265,  265,  265,  265,  265,  265,
      265,  265,  265,  265,  265,  266,  266,  266,  266,  266,
      266,  266,  266,  266,  266,  266,  267,  196,  190,  188,
      187,  267,  186,  267,  185,  267,  267,  268,  268,  184,
      268,  268,  268,  268,  268,  268,  268,  268,  269,  269,

      269,  269,  269,  269,  269,  269,  269,  269,  269,  270,
      173,  171,  170,  169,  270,  167,  270,  165,  270,  270,
      271,  271,  163,  271,  271,  271,  162,  271,  271,  271,
      271,  272,  272,  158,  272,  272,  272,  272,  272,  272,
      272,  272,  273,  273,  273,  273,  273,  273,  273,  273,
      273,  273,  273,  274,  274,  156,  274,  274,  274,  274,
      274,  274,  274,  274,  275,  275,  275,  275,  275,  275,
      275,  275,  275,  275,  275,  276,  155,  154,  146,  145,
      276,  276,  276,  276,  276,  276,  277,  277,  277,  277,
      277,  277,  277,  277,  277,  277,  277,  278,  278,  278,

      278,  278,  278,  278,  278,  278,  278,  278,  279,  279,
      279,  279,  279,  279,  279,  279,  279,  279,  279,  280,
      144,  143,  142,  280,  280,  141,  280,  140,  280,  280,
      282,  282,  282,  138,  136,  282,  133,  282,  282,  282,
      282,  283,  130,  129,  283,  128,  283,  284,  127,  126,
      118,  117,  284,  284,  285,  285,  285,  285,  285,  285,
      285,  285,  285,  285,  285,  286,  286,  286,  286,  286,
      286,  286,  286,  286,  286,  286,  287,  287,  116,  115,
      287,  114,  287,  287,  287,  288,  288,  288,  288,  288,
      288,  288,  288,  288,  288,  288,  289,  289,  289,  289,

      289,  289,  289,  289,  289,  289,  289,  290,  290,  111,
      290,  290,  290,  290,  290,  290,  290,  290,  291,  291,
      291,  291,  291,  291,  291,  291,  291,  291,  291,  292,
      292,  292,  292,  292,  292,  292,  292,  292,  292,  292,
      293,  293,  293,  293,  293,  293,  293,  293,  293,  293,
      293,  294,  294,  294,  294,  294,  294,  294,  294,  294,
      294,  294,  295,  295,  295,  295,  295,  295,  295,  295,
      295,  295,  295,  296,  296,  296,  296,  296,  296,  296,
      296,  296,  296,  296,  297,  297,  297,  297,  297,  297,
      297,  297,  297,  297,  297,  298,  298,  298,  298,  298,

      298,  298,  298,  298,  298,  298,  299,  299,  299,  299,
      299,  299,  299,  299,  299,  299,  299,  300,  300,  300,
      300,  300,  300,  300,  300,  300,  300,  300,  301,  301,
      301,  301,  301,  301,  301,  301,  301,  301,  301,  302,
      302,  302,  302,  302,  302,  302,  302,  302,  302,  302,
      303,  303,  303,  303,  303,  303,  303,  303,  303,  303,
      303,  304,  304,  304,  304,  304,  304,  304,  304,  304,
      304,  304,  305,  305,  305,  305,  305,  305,  305,  305,
      305,  305,  305,  306,  306,  306,  306,  306,  306,  306,
      306,  306,  306,  306,  105,  103,  102,  101,  100,   99,

       90,   89,   88,   87,   86,   78,   69,   64,   59,   58,
       57,   56,   55,   49,   45,   43,   33,   27,   21,   11,
        5,  261,  261,  261,  261,  261,  261,  261,  261,  261,
      261,  261,  261,  261,  261,  261,  261,  261,  261,  261,
      261,  261,  261,  261,  261,  261,  261,  261,  261,  261,
      261,  261,  261
    } ;

static grecs_grecs__state_type grecs_grecs__last_accepting_state;
static char *grecs_grecs__last_accepting_cpos;

extern int grecs_grecs__flex_debug;
int grecs_grecs__flex_debug = 1;

static const flex_int16_t grecs_grecs__rule_linenum[41] =
    {   0,
      190,  196,  197,  198,  200,  217,  221,  227,  229,  232,
      235,  242,  244,  245,  248,  251,  257,  264,  272,  276,
      283,  291,  301,  309,  316,  323,  328,  332,  347,  348,
      349,  350,  351,  352,  359,  362,  394,  396,  400,  404
    } ;

/* The intent behind this definition is that it'll catch
 * any uses of REJECT which flex missed.
 */
#define REJECT reject_used_but_not_detected
#define grecs_grecs_more() grecs_grecs_more_used_but_not_detected
#define YY_MORE_ADJ 0
#define YY_RESTORE_YY_MORE_OFFSET
char *grecs_grecs_text;
#line 1 "grecs-lex.l"
/* grecs - Gray's Extensible Configuration System       -*- c -*- */

#line 9 "grecs-lex.l"
/* grecs - Gray's Extensible Configuration System
   Copyright (C) 2007-2022 Sergey Poznyakoff

   Grecs is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 3 of the License, or (at your
   option) any later version.

   Grecs is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with Grecs. If not, see <http://www.gnu.org/licenses/>. */

#include <grecs.h>
#include <grecs-gram.h>
#include <unistd.h>
#include <fcntl.h>
#include <ctype.h>
#include <stdlib.h>
#include <errno.h>
#include <sys/stat.h>
#include <sys/wait.h>	
#include <glob.h>
#include <signal.h>
#include <wordsplit.h>

/*
 * Locus tracking
 */
struct grecs_locus_point grecs_current_locus_point;   /* Input file location */

#define YY_USER_ACTION do {						\
		if (YYSTATE == INITIAL) {		                \
			grecs_grecs_lloc.beg = grecs_current_locus_point;		\
			grecs_grecs_lloc.beg.col++;				\
		}							\
  		grecs_current_locus_point.col += grecs_grecs_leng;		\
 		grecs_grecs_lloc.end = grecs_current_locus_point;			\
   	} while (0);

/*
 * Redefine default error reporting function.
 */
#define YY_FATAL_ERROR(t) grecs_error(NULL, errno, "%s", t)

/*
 * Preprocessing and synclines.
 */
static int preproc_mode;
static int emit_syncline; /* If not null, emit syncline before next output
			   * line.
			   */
static int beginning_of_line; /* True if output is at the beginning of line */

/*
 * Echo LEN bytes from TEXT to grecs_grecs_out.
 */
static void
echo(char const *text, size_t len)
{
	if (len == 0) return;
	/* Check if we should print a syncline */
	if (beginning_of_line && emit_syncline) {
		fprintf(grecs_grecs_out, "#line %lu \"%s\"\n",
			(unsigned long)grecs_grecs_lloc.end.line,
			grecs_grecs_lloc.end.file);
		emit_syncline = 0;
	}
	/* Actually echo the text. */
	fwrite(text, len, 1, grecs_grecs_out);
	/* Determine the beginning of line status. */
	beginning_of_line = text[len-1] == '\n';
}
/* Inform flex that it should use echo. */
#define ECHO echo(grecs_grecs_text, grecs_grecs_leng)
#define PPECHO do { if (preproc_mode) ECHO; } while(0)		

/*
 * String concatenation support.
 *
 * String concatenation is a legacy feature of Grecs parser that instructs
 * it to concatenate adjacent quoted strings.  It is enabled if the
 * GRECS_OPTION_QUOTED_STRING_CONCAT bit is set in parser options.  This
 * bit causes the lexer to qualify each double-quoted string as QSTRING.
 */
#define qstring()							\
	((grecs_parser_options & GRECS_OPTION_QUOTED_STRING_CONCAT)	\
	 ? QSTRING : STRING)

/*
 * Lexical tie-in variables.
 */
extern int grecs_grecs_grecs_error_suppress;

/*
 * Here-document support.
 */
static char *multiline_delimiter;       /* Here-document delimiter text. */
static size_t multiline_delimiter_len;  /* Its length. */
static int multiline_unescape;          /* Unescape here-document contents */
static int (*char_to_strip)(char);      /* Strips matching characters of each
					   here-document line */

static void multiline_begin(char *);
static void multiline_add(char *);
static char *multiline_strip_tabs(char *text);
static int isemptystr(int off);

/*
 * Forward declarations.
 */
static int string_or_ident(void);
static void qstring_locus_fixup(void);
static int parse_include(char *text);
static int push_source(const char *name, int once);
static int pop_source(void);
static void ctx_abort(void);

/*
 * Versions of grecs up to 03044f768f didn't recognize comment starters that
 * followed a non-quoted string constituent, as in inet://127.0.0.1.  That
 * silently encouraged a bad practice of omitting quotes around such strings.
 *
 * To discontinue such practice and point to the source of the problem, a
 * warning is emitted each time a comment starter is encountered immediately
 * following a non-quoted string constituent.
 */
static int emit_comment_warning;
static void
comment_warning(void)
{
        if (emit_comment_warning) {
	        grecs_warning(&grecs_grecs_lloc, 0,
	                      _("\"%2.2s\" starts comment here; use quotes if you want it to be a part of string, or insert a whitespace to suppress this warning"), grecs_grecs_text);
	        emit_comment_warning = 0;
        }
}

/*
 * The report_diag should be called only when grecs_grecs_text begins with the following
 * prefix:
 *       ^[ \t]*#[ \t]*.{LEN}
 * It returns a pointer to grecs_grecs_text past the prefix.
 */
static char *
report_diag(size_t len)
{
	char *p = strchr(grecs_grecs_text, '#');
	p += strspn(p, " \t") + len;
	p += strspn(p, " \t");
	grecs_grecs_text[grecs_grecs_leng-1] = 0;
	if (p[0] == '"' && grecs_grecs_text[grecs_grecs_leng-2] == '"') {
		char *q, *r;
		
		grecs_grecs_text[grecs_grecs_leng-2] = 0;
		p++;
		q = r = p;
		do {
			if (*r == '\\') {
				int c = wordsplit_c_unquote_char(*++r);
				*q = c ? c : *r;
			} else
				*q = *r++;
		} while (*q++ == 0);
	}
	return p;
}
#line 1008 "grecs-lex.c"

#line 1010 "grecs-lex.c"

#define INITIAL 0
#define COMMENT 1
#define ML 2
#define STR 3
#define NQSTR 4

#ifndef YY_NO_UNISTD_H
/* Special case for "unistd.h", since it is non-ANSI. We include it way
 * down here because we want the user's section 1 to have been scanned first.
 * The user has a chance to override it with an option.
 */
/* %if-c-only */
#include <unistd.h>
/* %endif */
/* %if-c++-only */
/* %endif */
#endif

#ifndef YY_EXTRA_TYPE
#define YY_EXTRA_TYPE void *
#endif

/* %if-c-only Reentrant structure and macros (non-C++). */
/* %if-reentrant */
/* %if-c-only */

static int grecs_grecs__init_globals ( void );

/* %endif */
/* %if-reentrant */
/* %endif */
/* %endif End reentrant structures and macros. */

/* Accessor methods to globals.
   These are made visible to non-reentrant scanners for convenience. */

int grecs_grecs_lex_destroy ( void );

int grecs_grecs_get_debug ( void );

void grecs_grecs_set_debug ( int debug_flag  );

YY_EXTRA_TYPE grecs_grecs_get_extra ( void );

void grecs_grecs_set_extra ( YY_EXTRA_TYPE user_defined  );

FILE *grecs_grecs_get_in ( void );

void grecs_grecs_set_in  ( FILE * _in_str  );

FILE *grecs_grecs_get_out ( void );

void grecs_grecs_set_out  ( FILE * _out_str  );

			int grecs_grecs_get_leng ( void );

char *grecs_grecs_get_text ( void );

int grecs_grecs_get_lineno ( void );

void grecs_grecs_set_lineno ( int _line_number  );

/* %if-bison-bridge */
/* %endif */

/* Macros after this point can all be overridden by user definitions in
 * section 1.
 */

#ifndef YY_SKIP_YYWRAP
#ifdef __cplusplus
extern "C" int grecs_grecs_wrap ( void );
#else
extern int grecs_grecs_wrap ( void );
#endif
#endif

/* %not-for-header */
#ifndef YY_NO_UNPUT
    
#endif
/* %ok-for-header */

/* %endif */

#ifndef grecs_grecs_text_ptr
static void grecs_grecs__flex_strncpy ( char *, const char *, int );
#endif

#ifdef YY_NEED_STRLEN
static int grecs_grecs__flex_strlen ( const char * );
#endif

#ifndef YY_NO_INPUT
/* %if-c-only Standard (non-C++) definition */
/* %not-for-header */
#ifdef __cplusplus
static int grecs_grecs_input ( void );
#else
static int input ( void );
#endif
/* %ok-for-header */

/* %endif */
#endif

/* %if-c-only */

/* %endif */

/* Amount of stuff to slurp up with each read. */
#ifndef YY_READ_BUF_SIZE
#ifdef __ia64__
/* On IA-64, the buffer size is 16k, not 8k */
#define YY_READ_BUF_SIZE 16384
#else
#define YY_READ_BUF_SIZE 8192
#endif /* __ia64__ */
#endif

/* Copy whatever the last rule matched to the standard output. */
#ifndef ECHO
/* %if-c-only Standard (non-C++) definition */
/* This used to be an fputs(), but since the string might contain NUL's,
 * we now use fwrite().
 */
#define ECHO do { if (fwrite( grecs_grecs_text, (size_t) grecs_grecs_leng, 1, grecs_grecs_out )) {} } while (0)
/* %endif */
/* %if-c++-only C++ definition */
/* %endif */
#endif

/* Gets input and stuffs it into "buf".  number of characters read, or YY_NULL,
 * is returned in "result".
 */
#ifndef YY_INPUT
#define YY_INPUT(buf,result,max_size) \
/* %% [5.0] fread()/read() definition of YY_INPUT goes here unless we're doing C++ \ */\
	if ( YY_CURRENT_BUFFER_LVALUE->grecs_grecs__is_interactive ) \
		{ \
		int c = '*'; \
		int n; \
		for ( n = 0; n < max_size && \
			     (c = getc( grecs_grecs_in )) != EOF && c != '\n'; ++n ) \
			buf[n] = (char) c; \
		if ( c == '\n' ) \
			buf[n++] = (char) c; \
		if ( c == EOF && ferror( grecs_grecs_in ) ) \
			YY_FATAL_ERROR( "input in flex scanner failed" ); \
		result = n; \
		} \
	else \
		{ \
		errno=0; \
		while ( (result = (int) fread(buf, 1, (grecs_grecs__size_t) max_size, grecs_grecs_in)) == 0 && ferror(grecs_grecs_in)) \
			{ \
			if( errno != EINTR) \
				{ \
				YY_FATAL_ERROR( "input in flex scanner failed" ); \
				break; \
				} \
			errno=0; \
			clearerr(grecs_grecs_in); \
			} \
		}\
\
/* %if-c++-only C++ definition \ */\
/* %endif */

#endif

/* No semi-colon after return; correct usage is to write "grecs_grecs_terminate();" -
 * we don't want an extra ';' after the "return" because that will cause
 * some compilers to complain about unreachable statements.
 */
#ifndef grecs_grecs_terminate
#define grecs_grecs_terminate() return YY_NULL
#endif

/* Number of entries by which start-condition stack grows. */
#ifndef YY_START_STACK_INCR
#define YY_START_STACK_INCR 25
#endif

/* Report a fatal error. */
#ifndef YY_FATAL_ERROR
/* %if-c-only */
#define YY_FATAL_ERROR(msg) grecs_grecs__fatal_error( msg )
/* %endif */
/* %if-c++-only */
/* %endif */
#endif

/* %if-tables-serialization structures and prototypes */
/* %not-for-header */
/* %ok-for-header */

/* %not-for-header */
/* %tables-grecs_grecs_dmap generated elements */
/* %endif */
/* end tables serialization structures and prototypes */

/* %ok-for-header */

/* Default declaration of generated scanner - a define so the user can
 * easily add parameters.
 */
#ifndef YY_DECL
#define YY_DECL_IS_OURS 1
/* %if-c-only Standard (non-C++) definition */

extern int grecs_grecs_lex (void);

#define YY_DECL int grecs_grecs_lex (void)
/* %endif */
/* %if-c++-only C++ definition */
/* %endif */
#endif /* !YY_DECL */

/* Code executed at the beginning of each rule, after grecs_grecs_text and grecs_grecs_leng
 * have been set up.
 */
#ifndef YY_USER_ACTION
#define YY_USER_ACTION
#endif

/* Code executed at the end of each rule. */
#ifndef YY_BREAK
#define YY_BREAK /*LINTED*/break;
#endif

/* %% [6.0] YY_RULE_SETUP definition goes here */
#define YY_RULE_SETUP \
	if ( grecs_grecs_leng > 0 ) \
		YY_CURRENT_BUFFER_LVALUE->grecs_grecs__at_bol = \
				(grecs_grecs_text[grecs_grecs_leng - 1] == '\n'); \
	YY_USER_ACTION

/* %not-for-header */
/** The main scanner function which does all the work.
 */
YY_DECL
{
	grecs_grecs__state_type grecs_grecs__current_state;
	char *grecs_grecs__cp, *grecs_grecs__bp;
	int grecs_grecs__act;
    
	if ( !(grecs_grecs__init) )
		{
		(grecs_grecs__init) = 1;

#ifdef YY_USER_INIT
		YY_USER_INIT;
#endif

		if ( ! (grecs_grecs__start) )
			(grecs_grecs__start) = 1;	/* first start state */

		if ( ! grecs_grecs_in )
/* %if-c-only */
			grecs_grecs_in = stdin;
/* %endif */
/* %if-c++-only */
/* %endif */

		if ( ! grecs_grecs_out )
/* %if-c-only */
			grecs_grecs_out = stdout;
/* %endif */
/* %if-c++-only */
/* %endif */

		if ( ! YY_CURRENT_BUFFER ) {
			grecs_grecs_ensure_buffer_stack ();
			YY_CURRENT_BUFFER_LVALUE =
				grecs_grecs__create_buffer( grecs_grecs_in, YY_BUF_SIZE );
		}

		grecs_grecs__load_buffer_state(  );
		}

	{
/* %% [7.0] user's declarations go here */
#line 187 "grecs-lex.l"

#line 189 "grecs-lex.l"
         /* C-style comments */
#line 1299 "grecs-lex.c"

	while ( /*CONSTCOND*/1 )		/* loops until end-of-file is reached */
		{
/* %% [8.0] grecs_grecs_more()-related code goes here */
		grecs_grecs__cp = (grecs_grecs__c_buf_p);

		/* Support of grecs_grecs_text. */
		*grecs_grecs__cp = (grecs_grecs__hold_char);

		/* grecs_grecs__bp points to the position in grecs_grecs__ch_buf of the start of
		 * the current run.
		 */
		grecs_grecs__bp = grecs_grecs__cp;

/* %% [9.0] code to set up and find next match goes here */
		grecs_grecs__current_state = (grecs_grecs__start);
		grecs_grecs__current_state += YY_AT_BOL();
grecs_grecs__match:
		do
			{
			YY_CHAR grecs_grecs__c = grecs_grecs__ec[YY_SC_TO_UI(*grecs_grecs__cp)] ;
			if ( grecs_grecs__accept[grecs_grecs__current_state] )
				{
				(grecs_grecs__last_accepting_state) = grecs_grecs__current_state;
				(grecs_grecs__last_accepting_cpos) = grecs_grecs__cp;
				}
			while ( grecs_grecs__chk[grecs_grecs__base[grecs_grecs__current_state] + grecs_grecs__c] != grecs_grecs__current_state )
				{
				grecs_grecs__current_state = (int) grecs_grecs__def[grecs_grecs__current_state];
				if ( grecs_grecs__current_state >= 262 )
					grecs_grecs__c = grecs_grecs__meta[grecs_grecs__c];
				}
			grecs_grecs__current_state = grecs_grecs__nxt[grecs_grecs__base[grecs_grecs__current_state] + grecs_grecs__c];
			++grecs_grecs__cp;
			}
		while ( grecs_grecs__base[grecs_grecs__current_state] != 922 );

grecs_grecs__find_action:
/* %% [10.0] code to find the action number goes here */
		grecs_grecs__act = grecs_grecs__accept[grecs_grecs__current_state];
		if ( grecs_grecs__act == 0 )
			{ /* have to back up */
			grecs_grecs__cp = (grecs_grecs__last_accepting_cpos);
			grecs_grecs__current_state = (grecs_grecs__last_accepting_state);
			grecs_grecs__act = grecs_grecs__accept[grecs_grecs__current_state];
			}

		YY_DO_BEFORE_ACTION;

/* %% [11.0] code for grecs_grecs_lineno update goes here */

do_action:	/* This label is used only to access EOF actions. */

/* %% [12.0] debug code goes here */
		if ( grecs_grecs__flex_debug )
			{
			if ( grecs_grecs__act == 0 )
				fprintf( stderr, "--scanner backing up\n" );
			else if ( grecs_grecs__act < 41 )
				fprintf( stderr, "--accepting rule at line %ld (\"%s\")\n",
				         (long)grecs_grecs__rule_linenum[grecs_grecs__act], grecs_grecs_text );
			else if ( grecs_grecs__act == 41 )
				fprintf( stderr, "--accepting default rule (\"%s\")\n",
				         grecs_grecs_text );
			else if ( grecs_grecs__act == 42 )
				fprintf( stderr, "--(end of buffer or a NUL)\n" );
			else
				fprintf( stderr, "--EOF (start condition %d)\n", YY_START );
			}

		switch ( grecs_grecs__act )
	{ /* beginning of action switch */
/* %% [13.0] actions go here */
			case 0: /* must back up */
			/* undo the effects of YY_DO_BEFORE_ACTION */
			*grecs_grecs__cp = (grecs_grecs__hold_char);
			grecs_grecs__cp = (grecs_grecs__last_accepting_cpos);
			grecs_grecs__current_state = (grecs_grecs__last_accepting_state);
			goto grecs_grecs__find_action;

case 1:
YY_RULE_SETUP
#line 190 "grecs-lex.l"
{
            BEGIN(COMMENT);
	    beginning_of_line = 0;
	    comment_warning();
 }
	YY_BREAK

case 2:
YY_RULE_SETUP
#line 196 "grecs-lex.l"
/* eat anything that's not a '*' */
	YY_BREAK
case 3:
YY_RULE_SETUP
#line 197 "grecs-lex.l"
/* eat up '*'s not followed by '/'s */
	YY_BREAK
case 4:
/* rule 4 can match eol */
YY_RULE_SETUP
#line 198 "grecs-lex.l"
{ grecs_locus_point_advance_line(grecs_current_locus_point);
	      emit_syncline = 1; }
	YY_BREAK
case 5:
YY_RULE_SETUP
#line 200 "grecs-lex.l"
{
	      BEGIN(INITIAL);
	      if (preproc_mode) {
		      if (grecs_grecs_lloc.end.line != grecs_grecs_lloc.beg.line) {
			      if (!(grecs_grecs_lloc.beg.line == 1 && grecs_grecs_lloc.beg.col == 1))
				      echo("\n", 1);
			      beginning_of_line = 1;
		      } else
			      beginning_of_line = 0;
	      }
 }
	YY_BREAK
case YY_STATE_EOF(COMMENT):
#line 211 "grecs-lex.l"
{ grecs_error(&grecs_grecs_lloc, 0, _("end of file in comment"));
              return 0;
 }
	YY_BREAK

/* Line directive */
case 6:
/* rule 6 can match eol */
YY_RULE_SETUP
#line 217 "grecs-lex.l"
{
	 PPECHO;
         grecs_parse_line_directive_cpp(grecs_grecs_text, &grecs_grecs_lloc, &grecs_current_locus_point);
 }
	YY_BREAK
case 7:
/* rule 7 can match eol */
YY_RULE_SETUP
#line 221 "grecs-lex.l"
{
	 PPECHO;
	 grecs_parse_line_directive(grecs_grecs_text, &grecs_grecs_lloc, &grecs_current_locus_point);
 }
	YY_BREAK
/* Includes */
case 8:
/* rule 8 can match eol */
*grecs_grecs__cp = (grecs_grecs__hold_char); /* undo effects of setting up grecs_grecs_text */
YY_LINENO_REWIND_TO(grecs_grecs__cp - 1);
(grecs_grecs__c_buf_p) = grecs_grecs__cp -= 1;
YY_DO_BEFORE_ACTION; /* set up grecs_grecs_text again */
YY_RULE_SETUP
#line 227 "grecs-lex.l"
{ parse_include(grecs_grecs_text); }
	YY_BREAK
/* Error directives */
case 9:
/* rule 9 can match eol */
YY_RULE_SETUP
#line 229 "grecs-lex.l"
{
  grecs_error(&grecs_grecs_lloc, 0, "%s", report_diag(sizeof("error")));
 }
	YY_BREAK
case 10:
/* rule 10 can match eol */
YY_RULE_SETUP
#line 232 "grecs-lex.l"
{
  grecs_warning(&grecs_grecs_lloc, 0, "%s", report_diag(sizeof("warning")));
 }
	YY_BREAK
case 11:
/* rule 11 can match eol */
YY_RULE_SETUP
#line 235 "grecs-lex.l"
{
  grecs_error(&grecs_grecs_lloc, 0, "%s", report_diag(sizeof("abend")));
  ctx_abort();
  grecs_grecs_grecs_error_suppress = 1;
  return 0;
 }
	YY_BREAK
/* End-of-line comments */
case 12:
/* rule 12 can match eol */
YY_RULE_SETUP
#line 242 "grecs-lex.l"
{ grecs_locus_point_advance_line(grecs_current_locus_point);
            emit_syncline = 1; }
	YY_BREAK
case 13:
YY_RULE_SETUP
#line 244 "grecs-lex.l"
/* end-of-file comment */;
	YY_BREAK
case 14:
/* rule 14 can match eol */
YY_RULE_SETUP
#line 245 "grecs-lex.l"
{ comment_warning();
            grecs_locus_point_advance_line(grecs_current_locus_point);
            emit_syncline = 1; }
	YY_BREAK
case 15:
YY_RULE_SETUP
#line 248 "grecs-lex.l"
{ comment_warning(); } /* end of file comment */ 
	YY_BREAK
/* Non-quoted strings and identifiers */
case 16:
YY_RULE_SETUP
#line 251 "grecs-lex.l"
{
	 PPECHO;
         grecs_line_begin();
	 grecs_line_add(grecs_grecs_text, grecs_grecs_leng);
	 BEGIN(NQSTR);
 }
	YY_BREAK
case 17:
YY_RULE_SETUP
#line 257 "grecs-lex.l"
{
	 PPECHO;
         grecs_line_begin();
	 grecs_line_add(grecs_grecs_text, grecs_grecs_leng);
	 BEGIN(NQSTR);
 }
	YY_BREAK

case 18:
*grecs_grecs__cp = (grecs_grecs__hold_char); /* undo effects of setting up grecs_grecs_text */
(grecs_grecs__c_buf_p) = grecs_grecs__cp -= 2;
YY_DO_BEFORE_ACTION; /* set up grecs_grecs_text again */
YY_RULE_SETUP
#line 264 "grecs-lex.l"
{
	 PPECHO;
         emit_comment_warning = 1;
         grecs_line_add(grecs_grecs_text, grecs_grecs_leng);
         grecs_grecs_lval.string = grecs_line_finish();
	 BEGIN(INITIAL);
         return STRING;
 }
	YY_BREAK
case 19:
YY_RULE_SETUP
#line 272 "grecs-lex.l"
{
         PPECHO;
         grecs_line_add(grecs_grecs_text, grecs_grecs_leng);
 }
	YY_BREAK
case 20:
YY_RULE_SETUP
#line 276 "grecs-lex.l"
{ 
         PPECHO;
         grecs_line_add(grecs_grecs_text, grecs_grecs_leng);
         grecs_grecs_lval.string = grecs_line_finish();
	 BEGIN(INITIAL);
         return string_or_ident();
 }
	YY_BREAK
case 21:
/* rule 21 can match eol */
YY_RULE_SETUP
#line 283 "grecs-lex.l"
{
         grecs_grecs_less(0);
         grecs_grecs_lloc.end.col--;
         grecs_current_locus_point.col--;
         grecs_grecs_lval.string = grecs_line_finish();
	 BEGIN(INITIAL);
         return string_or_ident();
 }	 
	YY_BREAK
case 22:
YY_RULE_SETUP
#line 291 "grecs-lex.l"
{
         grecs_grecs_less(0);
         grecs_grecs_lloc.end.col--;
         grecs_current_locus_point.col--;
         grecs_grecs_lval.string = grecs_line_finish();
	 BEGIN(INITIAL);
         return string_or_ident();
 }	 
	YY_BREAK

/* Quoted strings */
case 23:
YY_RULE_SETUP
#line 301 "grecs-lex.l"
{
         PPECHO;
         grecs_line_begin();
         grecs_line_add(grecs_grecs_text + 1, grecs_grecs_leng - 2);
         grecs_grecs_lval.string = grecs_line_finish();
         qstring_locus_fixup();
         return qstring();
 }
	YY_BREAK
case 24:
/* rule 24 can match eol */
YY_RULE_SETUP
#line 309 "grecs-lex.l"
{
         PPECHO;
	 BEGIN(STR);
         grecs_line_begin();
 	 grecs_line_acc_grow_unescape_last(grecs_grecs_text + 1, grecs_grecs_leng - 1, &grecs_grecs_lloc);
         grecs_locus_point_advance_line(grecs_current_locus_point);
 }
	YY_BREAK
case 25:
YY_RULE_SETUP
#line 316 "grecs-lex.l"
{
         PPECHO;
         BEGIN(STR);
         grecs_line_begin();
	 grecs_line_acc_grow_unescape_last(grecs_grecs_text + 1, grecs_grecs_leng - 1, &grecs_grecs_lloc);
 }
	YY_BREAK

case 26:
/* rule 26 can match eol */
YY_RULE_SETUP
#line 323 "grecs-lex.l"
{
         PPECHO;
	 grecs_line_acc_grow_unescape_last(grecs_grecs_text, grecs_grecs_leng, &grecs_grecs_lloc);
         grecs_locus_point_advance_line(grecs_current_locus_point);
 }
	YY_BREAK
case 27:
YY_RULE_SETUP
#line 328 "grecs-lex.l"
{
         PPECHO;
         grecs_line_acc_grow_unescape_last(grecs_grecs_text, grecs_grecs_leng, &grecs_grecs_lloc);
 }
	YY_BREAK
case 28:
YY_RULE_SETUP
#line 332 "grecs-lex.l"
{
         PPECHO;
	 BEGIN(INITIAL);
         if (grecs_grecs_leng > 1) 
	         grecs_line_add(grecs_grecs_text, grecs_grecs_leng - 1); 
         grecs_grecs_lval.string = grecs_line_finish();
         qstring_locus_fixup();    
	 return qstring();
 }
	YY_BREAK
case YY_STATE_EOF(STR):
#line 341 "grecs-lex.l"
{
	grecs_error(&grecs_grecs_lloc, 0, _("end of file in quoted string"));
        return 0;
 }
	YY_BREAK

/* Multiline strings */
case 29:
/* rule 29 can match eol */
#line 348 "grecs-lex.l"
case 30:
/* rule 30 can match eol */
#line 349 "grecs-lex.l"
case 31:
/* rule 31 can match eol */
#line 350 "grecs-lex.l"
case 32:
/* rule 32 can match eol */
#line 351 "grecs-lex.l"
case 33:
/* rule 33 can match eol */
#line 352 "grecs-lex.l"
case 34:
/* rule 34 can match eol */
YY_RULE_SETUP
#line 352 "grecs-lex.l"
{
         PPECHO;
         BEGIN(ML);
	 multiline_begin(grecs_grecs_text+2);
 }
	YY_BREAK

/* Ignore m4 line statements */
case 35:
/* rule 35 can match eol */
YY_RULE_SETUP
#line 359 "grecs-lex.l"
{
	grecs_locus_point_advance_line(grecs_current_locus_point);
}
	YY_BREAK
case 36:
/* rule 36 can match eol */
YY_RULE_SETUP
#line 362 "grecs-lex.l"
{   char *p = multiline_strip_tabs(grecs_grecs_text);
         int len = p + multiline_delimiter_len - grecs_grecs_text;
         if (!strncmp(p, multiline_delimiter, multiline_delimiter_len)
             && isemptystr(len)) {
                 /* Update end pos */
		 if (grecs_parser_options & GRECS_OPTION_ADJUST_STRING_LOCATIONS) 
			 grecs_grecs_lloc.end.col = 1;
		 else
			 grecs_grecs_lloc.end.col -= grecs_grecs_leng - len;
		 /*
		  * Adjust current position and put the characters that
		  * follow the delimiter back to input.
		  */
                 grecs_current_locus_point.col -= grecs_grecs_leng - len;		 
  		 grecs_grecs_less(len);

		 /* Echo the consumed input if preprocessing. */
		 PPECHO;
		 
	         grecs_free(multiline_delimiter);
	         multiline_delimiter = NULL;
	         BEGIN(INITIAL);
	         grecs_grecs_lval.string = grecs_line_finish();

	         return MSTRING;
         } else
		 PPECHO;		  
         grecs_locus_point_advance_line(grecs_current_locus_point);
         multiline_add(p);
  }
	YY_BREAK

case 37:
YY_RULE_SETUP
#line 394 "grecs-lex.l"
PPECHO;
	YY_BREAK
/* Other tokens */
case 38:
/* rule 38 can match eol */
YY_RULE_SETUP
#line 396 "grecs-lex.l"
{
         PPECHO;
	 grecs_locus_point_advance_line(grecs_current_locus_point);
 } 
	YY_BREAK
case 39:
YY_RULE_SETUP
#line 400 "grecs-lex.l"
{
         PPECHO;
	 return grecs_grecs_text[0];
 }
	YY_BREAK
case 40:
YY_RULE_SETUP
#line 404 "grecs-lex.l"
{ if (isascii(grecs_grecs_text[0]) && isprint(grecs_grecs_text[0]))
		grecs_error(&grecs_grecs_lloc, 0,
			     _("stray character %c"), grecs_grecs_text[0]);
           else
		grecs_error(&grecs_grecs_lloc, 0,
			     _("stray character \\%03o"),
			       (unsigned char) grecs_grecs_text[0]); }
	YY_BREAK
case 41:
YY_RULE_SETUP
#line 412 "grecs-lex.l"
ECHO;
	YY_BREAK
#line 1781 "grecs-lex.c"
case YY_STATE_EOF(INITIAL):
case YY_STATE_EOF(ML):
case YY_STATE_EOF(NQSTR):
	grecs_grecs_terminate();

	case YY_END_OF_BUFFER:
		{
		/* Amount of text matched not including the EOB char. */
		int grecs_grecs__amount_of_matched_text = (int) (grecs_grecs__cp - (grecs_grecs_text_ptr)) - 1;

		/* Undo the effects of YY_DO_BEFORE_ACTION. */
		*grecs_grecs__cp = (grecs_grecs__hold_char);
		YY_RESTORE_YY_MORE_OFFSET

		if ( YY_CURRENT_BUFFER_LVALUE->grecs_grecs__buffer_status == YY_BUFFER_NEW )
			{
			/* We're scanning a new file or input source.  It's
			 * possible that this happened because the user
			 * just pointed grecs_grecs_in at a new source and called
			 * grecs_grecs_lex().  If so, then we have to assure
			 * consistency between YY_CURRENT_BUFFER and our
			 * globals.  Here is the right place to do so, because
			 * this is the first action (other than possibly a
			 * back-up) that will match for the new input source.
			 */
			(grecs_grecs__n_chars) = YY_CURRENT_BUFFER_LVALUE->grecs_grecs__n_chars;
/* %if-c-only */
			YY_CURRENT_BUFFER_LVALUE->grecs_grecs__input_file = grecs_grecs_in;
/* %endif */
/* %if-c++-only */
/* %endif */
			YY_CURRENT_BUFFER_LVALUE->grecs_grecs__buffer_status = YY_BUFFER_NORMAL;
			}

		/* Note that here we test for grecs_grecs__c_buf_p "<=" to the position
		 * of the first EOB in the buffer, since grecs_grecs__c_buf_p will
		 * already have been incremented past the NUL character
		 * (since all states make transitions on EOB to the
		 * end-of-buffer state).  Contrast this with the test
		 * in input().
		 */
		if ( (grecs_grecs__c_buf_p) <= &YY_CURRENT_BUFFER_LVALUE->grecs_grecs__ch_buf[(grecs_grecs__n_chars)] )
			{ /* This was really a NUL. */
			grecs_grecs__state_type grecs_grecs__next_state;

			(grecs_grecs__c_buf_p) = (grecs_grecs_text_ptr) + grecs_grecs__amount_of_matched_text;

			grecs_grecs__current_state = grecs_grecs__get_previous_state(  );

			/* Okay, we're now positioned to make the NUL
			 * transition.  We couldn't have
			 * grecs_grecs__get_previous_state() go ahead and do it
			 * for us because it doesn't know how to deal
			 * with the possibility of jamming (and we don't
			 * want to build jamming into it because then it
			 * will run more slowly).
			 */

			grecs_grecs__next_state = grecs_grecs__try_NUL_trans( grecs_grecs__current_state );

			grecs_grecs__bp = (grecs_grecs_text_ptr) + YY_MORE_ADJ;

			if ( grecs_grecs__next_state )
				{
				/* Consume the NUL. */
				grecs_grecs__cp = ++(grecs_grecs__c_buf_p);
				grecs_grecs__current_state = grecs_grecs__next_state;
				goto grecs_grecs__match;
				}

			else
				{
/* %% [14.0] code to do back-up for compressed tables and set up grecs_grecs__cp goes here */
				grecs_grecs__cp = (grecs_grecs__c_buf_p);
				goto grecs_grecs__find_action;
				}
			}

		else switch ( grecs_grecs__get_next_buffer(  ) )
			{
			case EOB_ACT_END_OF_FILE:
				{
				(grecs_grecs__did_buffer_switch_on_eof) = 0;

				if ( grecs_grecs_wrap(  ) )
					{
					/* Note: because we've taken care in
					 * grecs_grecs__get_next_buffer() to have set up
					 * grecs_grecs_text, we can now set up
					 * grecs_grecs__c_buf_p so that if some total
					 * hoser (like flex itself) wants to
					 * call the scanner after we return the
					 * YY_NULL, it'll still work - another
					 * YY_NULL will get returned.
					 */
					(grecs_grecs__c_buf_p) = (grecs_grecs_text_ptr) + YY_MORE_ADJ;

					grecs_grecs__act = YY_STATE_EOF(YY_START);
					goto do_action;
					}

				else
					{
					if ( ! (grecs_grecs__did_buffer_switch_on_eof) )
						YY_NEW_FILE;
					}
				break;
				}

			case EOB_ACT_CONTINUE_SCAN:
				(grecs_grecs__c_buf_p) =
					(grecs_grecs_text_ptr) + grecs_grecs__amount_of_matched_text;

				grecs_grecs__current_state = grecs_grecs__get_previous_state(  );

				grecs_grecs__cp = (grecs_grecs__c_buf_p);
				grecs_grecs__bp = (grecs_grecs_text_ptr) + YY_MORE_ADJ;
				goto grecs_grecs__match;

			case EOB_ACT_LAST_MATCH:
				(grecs_grecs__c_buf_p) =
				&YY_CURRENT_BUFFER_LVALUE->grecs_grecs__ch_buf[(grecs_grecs__n_chars)];

				grecs_grecs__current_state = grecs_grecs__get_previous_state(  );

				grecs_grecs__cp = (grecs_grecs__c_buf_p);
				grecs_grecs__bp = (grecs_grecs_text_ptr) + YY_MORE_ADJ;
				goto grecs_grecs__find_action;
			}
		break;
		}

	default:
		YY_FATAL_ERROR(
			"fatal flex scanner internal error--no action found" );
	} /* end of action switch */
		} /* end of scanning one token */
	} /* end of user's declarations */
} /* end of grecs_grecs_lex */
/* %ok-for-header */

/* %if-c++-only */
/* %not-for-header */
/* %ok-for-header */

/* %endif */

/* grecs_grecs__get_next_buffer - try to read in a new buffer
 *
 * Returns a code representing an action:
 *	EOB_ACT_LAST_MATCH -
 *	EOB_ACT_CONTINUE_SCAN - continue scanning from current position
 *	EOB_ACT_END_OF_FILE - end of file
 */
/* %if-c-only */
static int grecs_grecs__get_next_buffer (void)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    	char *dest = YY_CURRENT_BUFFER_LVALUE->grecs_grecs__ch_buf;
	char *source = (grecs_grecs_text_ptr);
	int number_to_move, i;
	int ret_val;

	if ( (grecs_grecs__c_buf_p) > &YY_CURRENT_BUFFER_LVALUE->grecs_grecs__ch_buf[(grecs_grecs__n_chars) + 1] )
		YY_FATAL_ERROR(
		"fatal flex scanner internal error--end of buffer missed" );

	if ( YY_CURRENT_BUFFER_LVALUE->grecs_grecs__fill_buffer == 0 )
		{ /* Don't try to fill the buffer, so this is an EOF. */
		if ( (grecs_grecs__c_buf_p) - (grecs_grecs_text_ptr) - YY_MORE_ADJ == 1 )
			{
			/* We matched a single character, the EOB, so
			 * treat this as a final EOF.
			 */
			return EOB_ACT_END_OF_FILE;
			}

		else
			{
			/* We matched some text prior to the EOB, first
			 * process it.
			 */
			return EOB_ACT_LAST_MATCH;
			}
		}

	/* Try to read more data. */

	/* First move last chars to start of buffer. */
	number_to_move = (int) ((grecs_grecs__c_buf_p) - (grecs_grecs_text_ptr) - 1);

	for ( i = 0; i < number_to_move; ++i )
		*(dest++) = *(source++);

	if ( YY_CURRENT_BUFFER_LVALUE->grecs_grecs__buffer_status == YY_BUFFER_EOF_PENDING )
		/* don't do the read, it's not guaranteed to return an EOF,
		 * just force an EOF
		 */
		YY_CURRENT_BUFFER_LVALUE->grecs_grecs__n_chars = (grecs_grecs__n_chars) = 0;

	else
		{
			int num_to_read =
			YY_CURRENT_BUFFER_LVALUE->grecs_grecs__buf_size - number_to_move - 1;

		while ( num_to_read <= 0 )
			{ /* Not enough room in the buffer - grow it. */

			/* just a shorter name for the current buffer */
			YY_BUFFER_STATE b = YY_CURRENT_BUFFER_LVALUE;

			int grecs_grecs__c_buf_p_offset =
				(int) ((grecs_grecs__c_buf_p) - b->grecs_grecs__ch_buf);

			if ( b->grecs_grecs__is_our_buffer )
				{
				int new_size = b->grecs_grecs__buf_size * 2;

				if ( new_size <= 0 )
					b->grecs_grecs__buf_size += b->grecs_grecs__buf_size / 8;
				else
					b->grecs_grecs__buf_size *= 2;

				b->grecs_grecs__ch_buf = (char *)
					/* Include room in for 2 EOB chars. */
					grecs_grecs_realloc( (void *) b->grecs_grecs__ch_buf,
							 (grecs_grecs__size_t) (b->grecs_grecs__buf_size + 2)  );
				}
			else
				/* Can't grow it, we don't own it. */
				b->grecs_grecs__ch_buf = NULL;

			if ( ! b->grecs_grecs__ch_buf )
				YY_FATAL_ERROR(
				"fatal error - scanner input buffer overflow" );

			(grecs_grecs__c_buf_p) = &b->grecs_grecs__ch_buf[grecs_grecs__c_buf_p_offset];

			num_to_read = YY_CURRENT_BUFFER_LVALUE->grecs_grecs__buf_size -
						number_to_move - 1;

			}

		if ( num_to_read > YY_READ_BUF_SIZE )
			num_to_read = YY_READ_BUF_SIZE;

		/* Read in more data. */
		YY_INPUT( (&YY_CURRENT_BUFFER_LVALUE->grecs_grecs__ch_buf[number_to_move]),
			(grecs_grecs__n_chars), num_to_read );

		YY_CURRENT_BUFFER_LVALUE->grecs_grecs__n_chars = (grecs_grecs__n_chars);
		}

	if ( (grecs_grecs__n_chars) == 0 )
		{
		if ( number_to_move == YY_MORE_ADJ )
			{
			ret_val = EOB_ACT_END_OF_FILE;
			grecs_grecs_restart( grecs_grecs_in  );
			}

		else
			{
			ret_val = EOB_ACT_LAST_MATCH;
			YY_CURRENT_BUFFER_LVALUE->grecs_grecs__buffer_status =
				YY_BUFFER_EOF_PENDING;
			}
		}

	else
		ret_val = EOB_ACT_CONTINUE_SCAN;

	if (((grecs_grecs__n_chars) + number_to_move) > YY_CURRENT_BUFFER_LVALUE->grecs_grecs__buf_size) {
		/* Extend the array by 50%, plus the number we really need. */
		int new_size = (grecs_grecs__n_chars) + number_to_move + ((grecs_grecs__n_chars) >> 1);
		YY_CURRENT_BUFFER_LVALUE->grecs_grecs__ch_buf = (char *) grecs_grecs_realloc(
			(void *) YY_CURRENT_BUFFER_LVALUE->grecs_grecs__ch_buf, (grecs_grecs__size_t) new_size  );
		if ( ! YY_CURRENT_BUFFER_LVALUE->grecs_grecs__ch_buf )
			YY_FATAL_ERROR( "out of dynamic memory in grecs_grecs__get_next_buffer()" );
		/* "- 2" to take care of EOB's */
		YY_CURRENT_BUFFER_LVALUE->grecs_grecs__buf_size = (int) (new_size - 2);
	}

	(grecs_grecs__n_chars) += number_to_move;
	YY_CURRENT_BUFFER_LVALUE->grecs_grecs__ch_buf[(grecs_grecs__n_chars)] = YY_END_OF_BUFFER_CHAR;
	YY_CURRENT_BUFFER_LVALUE->grecs_grecs__ch_buf[(grecs_grecs__n_chars) + 1] = YY_END_OF_BUFFER_CHAR;

	(grecs_grecs_text_ptr) = &YY_CURRENT_BUFFER_LVALUE->grecs_grecs__ch_buf[0];

	return ret_val;
}

/* grecs_grecs__get_previous_state - get the state just before the EOB char was reached */

/* %if-c-only */
/* %not-for-header */
    static grecs_grecs__state_type grecs_grecs__get_previous_state (void)
/* %endif */
/* %if-c++-only */
/* %endif */
{
	grecs_grecs__state_type grecs_grecs__current_state;
	char *grecs_grecs__cp;
    
/* %% [15.0] code to get the start state into grecs_grecs__current_state goes here */
	grecs_grecs__current_state = (grecs_grecs__start);
	grecs_grecs__current_state += YY_AT_BOL();

	for ( grecs_grecs__cp = (grecs_grecs_text_ptr) + YY_MORE_ADJ; grecs_grecs__cp < (grecs_grecs__c_buf_p); ++grecs_grecs__cp )
		{
/* %% [16.0] code to find the next state goes here */
		YY_CHAR grecs_grecs__c = (*grecs_grecs__cp ? grecs_grecs__ec[YY_SC_TO_UI(*grecs_grecs__cp)] : 1);
		if ( grecs_grecs__accept[grecs_grecs__current_state] )
			{
			(grecs_grecs__last_accepting_state) = grecs_grecs__current_state;
			(grecs_grecs__last_accepting_cpos) = grecs_grecs__cp;
			}
		while ( grecs_grecs__chk[grecs_grecs__base[grecs_grecs__current_state] + grecs_grecs__c] != grecs_grecs__current_state )
			{
			grecs_grecs__current_state = (int) grecs_grecs__def[grecs_grecs__current_state];
			if ( grecs_grecs__current_state >= 262 )
				grecs_grecs__c = grecs_grecs__meta[grecs_grecs__c];
			}
		grecs_grecs__current_state = grecs_grecs__nxt[grecs_grecs__base[grecs_grecs__current_state] + grecs_grecs__c];
		}

	return grecs_grecs__current_state;
}

/* grecs_grecs__try_NUL_trans - try to make a transition on the NUL character
 *
 * synopsis
 *	next_state = grecs_grecs__try_NUL_trans( current_state );
 */
/* %if-c-only */
    static grecs_grecs__state_type grecs_grecs__try_NUL_trans  (grecs_grecs__state_type grecs_grecs__current_state )
/* %endif */
/* %if-c++-only */
/* %endif */
{
	int grecs_grecs__is_jam;
    /* %% [17.0] code to find the next state, and perhaps do backing up, goes here */
	char *grecs_grecs__cp = (grecs_grecs__c_buf_p);

	YY_CHAR grecs_grecs__c = 1;
	if ( grecs_grecs__accept[grecs_grecs__current_state] )
		{
		(grecs_grecs__last_accepting_state) = grecs_grecs__current_state;
		(grecs_grecs__last_accepting_cpos) = grecs_grecs__cp;
		}
	while ( grecs_grecs__chk[grecs_grecs__base[grecs_grecs__current_state] + grecs_grecs__c] != grecs_grecs__current_state )
		{
		grecs_grecs__current_state = (int) grecs_grecs__def[grecs_grecs__current_state];
		if ( grecs_grecs__current_state >= 262 )
			grecs_grecs__c = grecs_grecs__meta[grecs_grecs__c];
		}
	grecs_grecs__current_state = grecs_grecs__nxt[grecs_grecs__base[grecs_grecs__current_state] + grecs_grecs__c];
	grecs_grecs__is_jam = (grecs_grecs__current_state == 261);

		return grecs_grecs__is_jam ? 0 : grecs_grecs__current_state;
}

#ifndef YY_NO_UNPUT
/* %if-c-only */

/* %endif */
#endif

/* %if-c-only */
#ifndef YY_NO_INPUT
#ifdef __cplusplus
    static int grecs_grecs_input (void)
#else
    static int input  (void)
#endif

/* %endif */
/* %if-c++-only */
/* %endif */
{
	int c;
    
	*(grecs_grecs__c_buf_p) = (grecs_grecs__hold_char);

	if ( *(grecs_grecs__c_buf_p) == YY_END_OF_BUFFER_CHAR )
		{
		/* grecs_grecs__c_buf_p now points to the character we want to return.
		 * If this occurs *before* the EOB characters, then it's a
		 * valid NUL; if not, then we've hit the end of the buffer.
		 */
		if ( (grecs_grecs__c_buf_p) < &YY_CURRENT_BUFFER_LVALUE->grecs_grecs__ch_buf[(grecs_grecs__n_chars)] )
			/* This was really a NUL. */
			*(grecs_grecs__c_buf_p) = '\0';

		else
			{ /* need more input */
			int offset = (int) ((grecs_grecs__c_buf_p) - (grecs_grecs_text_ptr));
			++(grecs_grecs__c_buf_p);

			switch ( grecs_grecs__get_next_buffer(  ) )
				{
				case EOB_ACT_LAST_MATCH:
					/* This happens because grecs_grecs__g_n_b()
					 * sees that we've accumulated a
					 * token and flags that we need to
					 * try matching the token before
					 * proceeding.  But for input(),
					 * there's no matching to consider.
					 * So convert the EOB_ACT_LAST_MATCH
					 * to EOB_ACT_END_OF_FILE.
					 */

					/* Reset buffer status. */
					grecs_grecs_restart( grecs_grecs_in );

					/*FALLTHROUGH*/

				case EOB_ACT_END_OF_FILE:
					{
					if ( grecs_grecs_wrap(  ) )
						return 0;

					if ( ! (grecs_grecs__did_buffer_switch_on_eof) )
						YY_NEW_FILE;
#ifdef __cplusplus
					return grecs_grecs_input();
#else
					return input();
#endif
					}

				case EOB_ACT_CONTINUE_SCAN:
					(grecs_grecs__c_buf_p) = (grecs_grecs_text_ptr) + offset;
					break;
				}
			}
		}

	c = *(unsigned char *) (grecs_grecs__c_buf_p);	/* cast for 8-bit char's */
	*(grecs_grecs__c_buf_p) = '\0';	/* preserve grecs_grecs_text */
	(grecs_grecs__hold_char) = *++(grecs_grecs__c_buf_p);

/* %% [19.0] update BOL and grecs_grecs_lineno */
	YY_CURRENT_BUFFER_LVALUE->grecs_grecs__at_bol = (c == '\n');

	return c;
}
/* %if-c-only */
#endif	/* ifndef YY_NO_INPUT */
/* %endif */

/** Immediately switch to a different input stream.
 * @param input_file A readable stream.
 * 
 * @note This function does not reset the start condition to @c INITIAL .
 */
/* %if-c-only */
    void grecs_grecs_restart  (FILE * input_file )
/* %endif */
/* %if-c++-only */
/* %endif */
{
    
	if ( ! YY_CURRENT_BUFFER ){
        grecs_grecs_ensure_buffer_stack ();
		YY_CURRENT_BUFFER_LVALUE =
            grecs_grecs__create_buffer( grecs_grecs_in, YY_BUF_SIZE );
	}

	grecs_grecs__init_buffer( YY_CURRENT_BUFFER, input_file );
	grecs_grecs__load_buffer_state(  );
}

/* %if-c++-only */
/* %endif */

/** Switch to a different input buffer.
 * @param new_buffer The new input buffer.
 * 
 */
/* %if-c-only */
    void grecs_grecs__switch_to_buffer  (YY_BUFFER_STATE  new_buffer )
/* %endif */
/* %if-c++-only */
/* %endif */
{
    
	/* TODO. We should be able to replace this entire function body
	 * with
	 *		grecs_grecs_pop_buffer_state();
	 *		grecs_grecs_push_buffer_state(new_buffer);
     */
	grecs_grecs_ensure_buffer_stack ();
	if ( YY_CURRENT_BUFFER == new_buffer )
		return;

	if ( YY_CURRENT_BUFFER )
		{
		/* Flush out information for old buffer. */
		*(grecs_grecs__c_buf_p) = (grecs_grecs__hold_char);
		YY_CURRENT_BUFFER_LVALUE->grecs_grecs__buf_pos = (grecs_grecs__c_buf_p);
		YY_CURRENT_BUFFER_LVALUE->grecs_grecs__n_chars = (grecs_grecs__n_chars);
		}

	YY_CURRENT_BUFFER_LVALUE = new_buffer;
	grecs_grecs__load_buffer_state(  );

	/* We don't actually know whether we did this switch during
	 * EOF (grecs_grecs_wrap()) processing, but the only time this flag
	 * is looked at is after grecs_grecs_wrap() is called, so it's safe
	 * to go ahead and always set it.
	 */
	(grecs_grecs__did_buffer_switch_on_eof) = 1;
}

/* %if-c-only */
static void grecs_grecs__load_buffer_state  (void)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    	(grecs_grecs__n_chars) = YY_CURRENT_BUFFER_LVALUE->grecs_grecs__n_chars;
	(grecs_grecs_text_ptr) = (grecs_grecs__c_buf_p) = YY_CURRENT_BUFFER_LVALUE->grecs_grecs__buf_pos;
/* %if-c-only */
	grecs_grecs_in = YY_CURRENT_BUFFER_LVALUE->grecs_grecs__input_file;
/* %endif */
/* %if-c++-only */
/* %endif */
	(grecs_grecs__hold_char) = *(grecs_grecs__c_buf_p);
}

/** Allocate and initialize an input buffer state.
 * @param file A readable stream.
 * @param size The character buffer size in bytes. When in doubt, use @c YY_BUF_SIZE.
 * 
 * @return the allocated buffer state.
 */
/* %if-c-only */
    YY_BUFFER_STATE grecs_grecs__create_buffer  (FILE * file, int  size )
/* %endif */
/* %if-c++-only */
/* %endif */
{
	YY_BUFFER_STATE b;
    
	b = (YY_BUFFER_STATE) grecs_grecs_alloc( sizeof( struct grecs_grecs__buffer_state )  );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in grecs_grecs__create_buffer()" );

	b->grecs_grecs__buf_size = size;

	/* grecs_grecs__ch_buf has to be 2 characters longer than the size given because
	 * we need to put in 2 end-of-buffer characters.
	 */
	b->grecs_grecs__ch_buf = (char *) grecs_grecs_alloc( (grecs_grecs__size_t) (b->grecs_grecs__buf_size + 2)  );
	if ( ! b->grecs_grecs__ch_buf )
		YY_FATAL_ERROR( "out of dynamic memory in grecs_grecs__create_buffer()" );

	b->grecs_grecs__is_our_buffer = 1;

	grecs_grecs__init_buffer( b, file );

	return b;
}

/* %if-c++-only */
/* %endif */

/** Destroy the buffer.
 * @param b a buffer created with grecs_grecs__create_buffer()
 * 
 */
/* %if-c-only */
    void grecs_grecs__delete_buffer (YY_BUFFER_STATE  b )
/* %endif */
/* %if-c++-only */
/* %endif */
{
    
	if ( ! b )
		return;

	if ( b == YY_CURRENT_BUFFER ) /* Not sure if we should pop here. */
		YY_CURRENT_BUFFER_LVALUE = (YY_BUFFER_STATE) 0;

	if ( b->grecs_grecs__is_our_buffer )
		grecs_grecs_free( (void *) b->grecs_grecs__ch_buf  );

	grecs_grecs_free( (void *) b  );
}

/* Initializes or reinitializes a buffer.
 * This function is sometimes called more than once on the same buffer,
 * such as during a grecs_grecs_restart() or at EOF.
 */
/* %if-c-only */
    static void grecs_grecs__init_buffer  (YY_BUFFER_STATE  b, FILE * file )
/* %endif */
/* %if-c++-only */
/* %endif */

{
	int oerrno = errno;
    
	grecs_grecs__flush_buffer( b );

/* %if-c-only */
	b->grecs_grecs__input_file = file;
/* %endif */
/* %if-c++-only */
/* %endif */
	b->grecs_grecs__fill_buffer = 1;

    /* If b is the current buffer, then grecs_grecs__init_buffer was _probably_
     * called from grecs_grecs_restart() or through grecs_grecs__get_next_buffer.
     * In that case, we don't want to reset the lineno or column.
     */
    if (b != YY_CURRENT_BUFFER){
        b->grecs_grecs__bs_lineno = 1;
        b->grecs_grecs__bs_column = 0;
    }

/* %if-c-only */

        b->grecs_grecs__is_interactive = file ? (isatty( fileno(file) ) > 0) : 0;
    
/* %endif */
/* %if-c++-only */
/* %endif */
	errno = oerrno;
}

/** Discard all buffered characters. On the next scan, YY_INPUT will be called.
 * @param b the buffer state to be flushed, usually @c YY_CURRENT_BUFFER.
 * 
 */
/* %if-c-only */
    void grecs_grecs__flush_buffer (YY_BUFFER_STATE  b )
/* %endif */
/* %if-c++-only */
/* %endif */
{
    	if ( ! b )
		return;

	b->grecs_grecs__n_chars = 0;

	/* We always need two end-of-buffer characters.  The first causes
	 * a transition to the end-of-buffer state.  The second causes
	 * a jam in that state.
	 */
	b->grecs_grecs__ch_buf[0] = YY_END_OF_BUFFER_CHAR;
	b->grecs_grecs__ch_buf[1] = YY_END_OF_BUFFER_CHAR;

	b->grecs_grecs__buf_pos = &b->grecs_grecs__ch_buf[0];

	b->grecs_grecs__at_bol = 1;
	b->grecs_grecs__buffer_status = YY_BUFFER_NEW;

	if ( b == YY_CURRENT_BUFFER )
		grecs_grecs__load_buffer_state(  );
}

/* %if-c-or-c++ */
/** Pushes the new state onto the stack. The new state becomes
 *  the current state. This function will allocate the stack
 *  if necessary.
 *  @param new_buffer The new state.
 *  
 */
/* %if-c-only */
void grecs_grecs_push_buffer_state (YY_BUFFER_STATE new_buffer )
/* %endif */
/* %if-c++-only */
/* %endif */
{
    	if (new_buffer == NULL)
		return;

	grecs_grecs_ensure_buffer_stack();

	/* This block is copied from grecs_grecs__switch_to_buffer. */
	if ( YY_CURRENT_BUFFER )
		{
		/* Flush out information for old buffer. */
		*(grecs_grecs__c_buf_p) = (grecs_grecs__hold_char);
		YY_CURRENT_BUFFER_LVALUE->grecs_grecs__buf_pos = (grecs_grecs__c_buf_p);
		YY_CURRENT_BUFFER_LVALUE->grecs_grecs__n_chars = (grecs_grecs__n_chars);
		}

	/* Only push if top exists. Otherwise, replace top. */
	if (YY_CURRENT_BUFFER)
		(grecs_grecs__buffer_stack_top)++;
	YY_CURRENT_BUFFER_LVALUE = new_buffer;

	/* copied from grecs_grecs__switch_to_buffer. */
	grecs_grecs__load_buffer_state(  );
	(grecs_grecs__did_buffer_switch_on_eof) = 1;
}
/* %endif */

/* %if-c-or-c++ */
/** Removes and deletes the top of the stack, if present.
 *  The next element becomes the new top.
 *  
 */
/* %if-c-only */
void grecs_grecs_pop_buffer_state (void)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    	if (!YY_CURRENT_BUFFER)
		return;

	grecs_grecs__delete_buffer(YY_CURRENT_BUFFER );
	YY_CURRENT_BUFFER_LVALUE = NULL;
	if ((grecs_grecs__buffer_stack_top) > 0)
		--(grecs_grecs__buffer_stack_top);

	if (YY_CURRENT_BUFFER) {
		grecs_grecs__load_buffer_state(  );
		(grecs_grecs__did_buffer_switch_on_eof) = 1;
	}
}
/* %endif */

/* %if-c-or-c++ */
/* Allocates the stack if it does not exist.
 *  Guarantees space for at least one push.
 */
/* %if-c-only */
static void grecs_grecs_ensure_buffer_stack (void)
/* %endif */
/* %if-c++-only */
/* %endif */
{
	grecs_grecs__size_t num_to_alloc;
    
	if (!(grecs_grecs__buffer_stack)) {

		/* First allocation is just for 2 elements, since we don't know if this
		 * scanner will even need a stack. We use 2 instead of 1 to avoid an
		 * immediate realloc on the next call.
         */
      num_to_alloc = 1; /* After all that talk, this was set to 1 anyways... */
		(grecs_grecs__buffer_stack) = (struct grecs_grecs__buffer_state**)grecs_grecs_alloc
								(num_to_alloc * sizeof(struct grecs_grecs__buffer_state*)
								);
		if ( ! (grecs_grecs__buffer_stack) )
			YY_FATAL_ERROR( "out of dynamic memory in grecs_grecs_ensure_buffer_stack()" );

		memset((grecs_grecs__buffer_stack), 0, num_to_alloc * sizeof(struct grecs_grecs__buffer_state*));

		(grecs_grecs__buffer_stack_max) = num_to_alloc;
		(grecs_grecs__buffer_stack_top) = 0;
		return;
	}

	if ((grecs_grecs__buffer_stack_top) >= ((grecs_grecs__buffer_stack_max)) - 1){

		/* Increase the buffer to prepare for a possible push. */
		grecs_grecs__size_t grow_size = 8 /* arbitrary grow size */;

		num_to_alloc = (grecs_grecs__buffer_stack_max) + grow_size;
		(grecs_grecs__buffer_stack) = (struct grecs_grecs__buffer_state**)grecs_grecs_realloc
								((grecs_grecs__buffer_stack),
								num_to_alloc * sizeof(struct grecs_grecs__buffer_state*)
								);
		if ( ! (grecs_grecs__buffer_stack) )
			YY_FATAL_ERROR( "out of dynamic memory in grecs_grecs_ensure_buffer_stack()" );

		/* zero only the new slots.*/
		memset((grecs_grecs__buffer_stack) + (grecs_grecs__buffer_stack_max), 0, grow_size * sizeof(struct grecs_grecs__buffer_state*));
		(grecs_grecs__buffer_stack_max) = num_to_alloc;
	}
}
/* %endif */

/* %if-c-only */
/** Setup the input buffer state to scan directly from a user-specified character buffer.
 * @param base the character buffer
 * @param size the size in bytes of the character buffer
 * 
 * @return the newly allocated buffer state object.
 */
YY_BUFFER_STATE grecs_grecs__scan_buffer  (char * base, grecs_grecs__size_t  size )
{
	YY_BUFFER_STATE b;
    
	if ( size < 2 ||
	     base[size-2] != YY_END_OF_BUFFER_CHAR ||
	     base[size-1] != YY_END_OF_BUFFER_CHAR )
		/* They forgot to leave room for the EOB's. */
		return NULL;

	b = (YY_BUFFER_STATE) grecs_grecs_alloc( sizeof( struct grecs_grecs__buffer_state )  );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in grecs_grecs__scan_buffer()" );

	b->grecs_grecs__buf_size = (int) (size - 2);	/* "- 2" to take care of EOB's */
	b->grecs_grecs__buf_pos = b->grecs_grecs__ch_buf = base;
	b->grecs_grecs__is_our_buffer = 0;
	b->grecs_grecs__input_file = NULL;
	b->grecs_grecs__n_chars = b->grecs_grecs__buf_size;
	b->grecs_grecs__is_interactive = 0;
	b->grecs_grecs__at_bol = 1;
	b->grecs_grecs__fill_buffer = 0;
	b->grecs_grecs__buffer_status = YY_BUFFER_NEW;

	grecs_grecs__switch_to_buffer( b  );

	return b;
}
/* %endif */

/* %if-c-only */
/** Setup the input buffer state to scan a string. The next call to grecs_grecs_lex() will
 * scan from a @e copy of @a str.
 * @param grecs_grecs_str a NUL-terminated string to scan
 * 
 * @return the newly allocated buffer state object.
 * @note If you want to scan bytes that may contain NUL values, then use
 *       grecs_grecs__scan_bytes() instead.
 */
YY_BUFFER_STATE grecs_grecs__scan_string (const char * grecs_grecs_str )
{
    
	return grecs_grecs__scan_bytes( grecs_grecs_str, (int) strlen(grecs_grecs_str) );
}
/* %endif */

/* %if-c-only */
/** Setup the input buffer state to scan the given bytes. The next call to grecs_grecs_lex() will
 * scan from a @e copy of @a bytes.
 * @param grecs_grecs_bytes the byte buffer to scan
 * @param _grecs_grecs_bytes_len the number of bytes in the buffer pointed to by @a bytes.
 * 
 * @return the newly allocated buffer state object.
 */
YY_BUFFER_STATE grecs_grecs__scan_bytes  (const char * grecs_grecs_bytes, int  _grecs_grecs_bytes_len )
{
	YY_BUFFER_STATE b;
	char *buf;
	grecs_grecs__size_t n;
	int i;
    
	/* Get memory for full buffer, including space for trailing EOB's. */
	n = (grecs_grecs__size_t) (_grecs_grecs_bytes_len + 2);
	buf = (char *) grecs_grecs_alloc( n  );
	if ( ! buf )
		YY_FATAL_ERROR( "out of dynamic memory in grecs_grecs__scan_bytes()" );

	for ( i = 0; i < _grecs_grecs_bytes_len; ++i )
		buf[i] = grecs_grecs_bytes[i];

	buf[_grecs_grecs_bytes_len] = buf[_grecs_grecs_bytes_len+1] = YY_END_OF_BUFFER_CHAR;

	b = grecs_grecs__scan_buffer( buf, n );
	if ( ! b )
		YY_FATAL_ERROR( "bad buffer in grecs_grecs__scan_bytes()" );

	/* It's okay to grow etc. this buffer, and we should throw it
	 * away when we're done.
	 */
	b->grecs_grecs__is_our_buffer = 1;

	return b;
}
/* %endif */

#ifndef YY_EXIT_FAILURE
#define YY_EXIT_FAILURE 2
#endif

/* %if-c-only */
static void grecs_grecs_noreturn grecs_grecs__fatal_error (const char* msg )
{
			fprintf( stderr, "%s\n", msg );
	exit( YY_EXIT_FAILURE );
}
/* %endif */
/* %if-c++-only */
/* %endif */

/* Redefine grecs_grecs_less() so it works in section 3 code. */

#undef grecs_grecs_less
#define grecs_grecs_less(n) \
	do \
		{ \
		/* Undo effects of setting up grecs_grecs_text. */ \
        int grecs_grecs_less_macro_arg = (n); \
        YY_LESS_LINENO(grecs_grecs_less_macro_arg);\
		grecs_grecs_text[grecs_grecs_leng] = (grecs_grecs__hold_char); \
		(grecs_grecs__c_buf_p) = grecs_grecs_text + grecs_grecs_less_macro_arg; \
		(grecs_grecs__hold_char) = *(grecs_grecs__c_buf_p); \
		*(grecs_grecs__c_buf_p) = '\0'; \
		grecs_grecs_leng = grecs_grecs_less_macro_arg; \
		} \
	while ( 0 )

/* Accessor  methods (get/set functions) to struct members. */

/* %if-c-only */
/* %if-reentrant */
/* %endif */

/** Get the current line number.
 * 
 */
int grecs_grecs_get_lineno  (void)
{
    
    return grecs_grecs_lineno;
}

/** Get the input stream.
 * 
 */
FILE *grecs_grecs_get_in  (void)
{
        return grecs_grecs_in;
}

/** Get the output stream.
 * 
 */
FILE *grecs_grecs_get_out  (void)
{
        return grecs_grecs_out;
}

/** Get the length of the current token.
 * 
 */
int grecs_grecs_get_leng  (void)
{
        return grecs_grecs_leng;
}

/** Get the current token.
 * 
 */

char *grecs_grecs_get_text  (void)
{
        return grecs_grecs_text;
}

/* %if-reentrant */
/* %endif */

/** Set the current line number.
 * @param _line_number line number
 * 
 */
void grecs_grecs_set_lineno (int  _line_number )
{
    
    grecs_grecs_lineno = _line_number;
}

/** Set the input stream. This does not discard the current
 * input buffer.
 * @param _in_str A readable stream.
 * 
 * @see grecs_grecs__switch_to_buffer
 */
void grecs_grecs_set_in (FILE *  _in_str )
{
        grecs_grecs_in = _in_str ;
}

void grecs_grecs_set_out (FILE *  _out_str )
{
        grecs_grecs_out = _out_str ;
}

int grecs_grecs_get_debug  (void)
{
        return grecs_grecs__flex_debug;
}

void grecs_grecs_set_debug (int  _bdebug )
{
        grecs_grecs__flex_debug = _bdebug ;
}

/* %endif */

/* %if-reentrant */
/* %if-bison-bridge */
/* %endif */
/* %endif if-c-only */

/* %if-c-only */
static int grecs_grecs__init_globals (void)
{
        /* Initialization is the same as for the non-reentrant scanner.
     * This function is called from grecs_grecs_lex_destroy(), so don't allocate here.
     */

    (grecs_grecs__buffer_stack) = NULL;
    (grecs_grecs__buffer_stack_top) = 0;
    (grecs_grecs__buffer_stack_max) = 0;
    (grecs_grecs__c_buf_p) = NULL;
    (grecs_grecs__init) = 0;
    (grecs_grecs__start) = 0;

/* Defined in main.c */
#ifdef YY_STDINIT
    grecs_grecs_in = stdin;
    grecs_grecs_out = stdout;
#else
    grecs_grecs_in = NULL;
    grecs_grecs_out = NULL;
#endif

    /* For future reference: Set errno on error, since we are called by
     * grecs_grecs_lex_init()
     */
    return 0;
}
/* %endif */

/* %if-c-only SNIP! this currently causes conflicts with the c++ scanner */
/* grecs_grecs_lex_destroy is for both reentrant and non-reentrant scanners. */
int grecs_grecs_lex_destroy  (void)
{
    
    /* Pop the buffer stack, destroying each element. */
	while(YY_CURRENT_BUFFER){
		grecs_grecs__delete_buffer( YY_CURRENT_BUFFER  );
		YY_CURRENT_BUFFER_LVALUE = NULL;
		grecs_grecs_pop_buffer_state();
	}

	/* Destroy the stack itself. */
	grecs_grecs_free((grecs_grecs__buffer_stack) );
	(grecs_grecs__buffer_stack) = NULL;

    /* Reset the globals. This is important in a non-reentrant scanner so the next time
     * grecs_grecs_lex() is called, initialization will occur. */
    grecs_grecs__init_globals( );

/* %if-reentrant */
/* %endif */
    return 0;
}
/* %endif */

/*
 * Internal utility routines.
 */

#ifndef grecs_grecs_text_ptr
static void grecs_grecs__flex_strncpy (char* s1, const char * s2, int n )
{
		
	int i;
	for ( i = 0; i < n; ++i )
		s1[i] = s2[i];
}
#endif

#ifdef YY_NEED_STRLEN
static int grecs_grecs__flex_strlen (const char * s )
{
	int n;
	for ( n = 0; s[n]; ++n )
		;

	return n;
}
#endif

void *grecs_grecs_alloc (grecs_grecs__size_t  size )
{
			return malloc(size);
}

void *grecs_grecs_realloc  (void * ptr, grecs_grecs__size_t  size )
{
		
	/* The cast to (char *) in the following accommodates both
	 * implementations that use char* generic pointers, and those
	 * that use void* generic pointers.  It works with the latter
	 * because both ANSI C and C++ allow castless assignment from
	 * any pointer type to void*, and deal with argument conversions
	 * as though doing an assignment.
	 */
	return realloc(ptr, size);
}

void grecs_grecs_free (void * ptr )
{
			free( (char *) ptr );	/* see grecs_grecs_realloc() for (char *) cast */
}

/* %if-tables-serialization definitions */
/* %define-grecs_grecs_tables   The name for this specific scanner's tables. */
#define YYTABLES_NAME "grecs_grecs_tables"
/* %endif */

/* %ok-for-header */

#line 412 "grecs-lex.l"

/*
 * Source inclusion support.
 */
static glob_t include_glob;
static size_t include_pos;
static int include_once;

static int
isglob(const char *s)
{
	return s[strcspn(s, "*?[")] != 0;
}

static int
parse_include(char *text)
{
	char *p;
	int rc = 1;
	int once;

	p = strchr(text, '#');
	p += strspn(p, " \t") + sizeof("include");
	if (strncmp(p, "_once", 5) == 0) {
		once = 1;
		p += 5;
	} else
		once = 0;
	p += strspn(p, " \t");
	
	if (!*p) {
		grecs_error(&grecs_grecs_lloc, 0, _("invalid include statement"));
		return 1;
	} else {
		size_t len;
		int allow_cwd;
		
		len = strlen (p);
		
		if (p[0] == '<' && p[len - 1] == '>') {
			allow_cwd = 0;
			p[len - 1] = 0;
			p++;
		} else if (p[0] == '"' && p[len - 1] == '"') {
			allow_cwd = 1;
			p[len - 1] = 0;
			p++;
		} else {
			grecs_error(&grecs_grecs_lloc, 0, _("invalid include statement"));
			return 1;
		}
		
		if (isglob(p)) {
			switch (glob(p, 0, NULL, &include_glob)) {
			case 0:
				include_pos = 0;
				include_once = once;
				break;
			case GLOB_NOSPACE:
				grecs_alloc_die();
			case GLOB_NOMATCH:
				break;
			default:
				grecs_error(&grecs_grecs_lloc, 0,  _("read error"));
			}
			p = NULL;
		} else if (p[0] != '/') {
			char *q = p;
			p = grecs_find_include_file(q, allow_cwd);
			if (!p)
				grecs_error(&grecs_grecs_lloc, 0,
					    _("%s: No such file or directory"),
					    q);
		}
 	}
  
	if (p)
		rc = push_source(p, once);
	else if (include_pos < include_glob.gl_pathc)
		rc = push_source(include_glob.gl_pathv[include_pos++], once);
	return rc;
}

/*
 * Include search path.
 */
static struct grecs_list *grecs_usr_include_path;
static struct grecs_list *grecs_std_include_path;

size_t
grecs_include_path_count(int flag)
{
    size_t count = 0;
    if (flag & GRECS_STD_INCLUDE)
	count += grecs_list_size(grecs_std_include_path);
    if (flag & GRECS_USR_INCLUDE)
	count += grecs_list_size(grecs_usr_include_path);
    return count;
}

static int
foreach_dir(struct grecs_list *list, int flag,
	    int (*fun)(int, const char *, void *), void *data)
{
    int rc = 0;
    struct grecs_list_entry *ep;

    for (ep = list->head; rc == 0 && ep; ep = ep->next)
	rc = fun(flag, ep->data, data);
    return rc;
}

int
grecs_foreach_include_dir(int flag, int (*fun)(int, const char *, void *),
			  void *data)
{
    int rc = 0;

    if (flag & GRECS_STD_INCLUDE)
	rc = foreach_dir(grecs_std_include_path, GRECS_STD_INCLUDE, fun, data);
    if (rc == 0 && (flag & GRECS_USR_INCLUDE))
	rc = foreach_dir(grecs_usr_include_path, GRECS_USR_INCLUDE, fun, data);
    return rc;
}

void
grecs_include_path_clear()
{
	if (grecs_usr_include_path)
		grecs_list_clear(grecs_usr_include_path);
	if (grecs_std_include_path)
		grecs_list_clear(grecs_std_include_path);
}

static void
incl_free(void *data)
{
	grecs_free(data);
}

void
grecs_include_path_setup_v(char **dirs)
{
	if (!grecs_usr_include_path) {
		grecs_usr_include_path = grecs_list_create();
		grecs_usr_include_path->free_entry = incl_free;
	}
	grecs_std_include_path = grecs_list_create();
	grecs_std_include_path->free_entry = incl_free;
	if (dirs) {
		int i;
		for (i = 0; dirs[i]; i++)
			/* FIXME: Element never freed */
			grecs_list_append(grecs_std_include_path,
					  grecs_strdup(dirs[i]));
	}
}

void
grecs_include_path_setup(const char *dir, ...)
{
	const char *p;
	char **argv = NULL;
	size_t argc = 0;
	size_t argi = 0;
	va_list ap;
	
	va_start(ap, dir);
	p = dir;
	while (1) {
		if (argi == argc) {
			if (argc == 0)
				argc = 16;
			else
				argc += 16;
			argv = grecs_realloc(argv, argc * sizeof(argv[0]));
		}
		argv[argi++] = (char*) p;
		if (!p)
			break;
		p = va_arg(ap, const char*);
	}
	grecs_include_path_setup_v(argv);
	grecs_free(argv);
	va_end(ap);
}

void
grecs_preproc_add_include_dir(char *dir)
{
	if (!grecs_usr_include_path) {
		grecs_usr_include_path = grecs_list_create();
		grecs_usr_include_path->free_entry = incl_free;
	}
	grecs_list_append(grecs_usr_include_path, grecs_strdup(dir));
}

struct file_data {
	const char *name;
	size_t namelen;
	char *buf;
	size_t buflen;
	int found;
};

static int
pp_list_find(struct grecs_list *list, struct file_data *dptr)
{
	struct grecs_list_entry *ep;

	if (!list)
		return 0;
	for (ep = list->head; !dptr->found && ep; ep = ep->next) {
		const char *dir = ep->data;
		size_t size = strlen (dir) + 1 + dptr->namelen + 1;
		if (size > dptr->buflen) {
			dptr->buflen = size;
			dptr->buf = grecs_realloc(dptr->buf, dptr->buflen);
		}
		strcpy(dptr->buf, dir);
		strcat(dptr->buf, "/");
		strcat(dptr->buf, dptr->name);
		dptr->found = access(dptr->buf, F_OK) == 0;
	}
	return dptr->found;
}

char *
grecs_find_include_file(const char *name, int allow_cwd)
{
	static char *cwd = ".";
	struct file_data fd;

	fd.name = name;
	fd.namelen = strlen(name);
	fd.buf = NULL;
	fd.buflen = 0;
	fd.found = 0;

	if (!grecs_usr_include_path)
		grecs_include_path_setup(NULL);
	if (allow_cwd) {
		grecs_list_append(grecs_usr_include_path, cwd);
		pp_list_find(grecs_usr_include_path, &fd);
		grecs_list_remove_tail(grecs_usr_include_path);
	} else
		pp_list_find(grecs_usr_include_path, &fd);

	if (!fd.found) {
		pp_list_find(grecs_std_include_path, &fd);
		if (!fd.found)
			return NULL;
	}
	return fd.buf;
}

/*
 * Input context support.
 */

struct input_file_ident {
	ino_t i_node;
	dev_t device;
	pid_t pid;
};

struct buffer_ctx {
	struct buffer_ctx *prev;	/* Pointer to previous context */
	struct grecs_locus locus;
	struct grecs_locus_point point;	/* Current input location */
	size_t namelen;		/* Length of the file name */
	struct input_file_ident id;
	FILE *infile;
	YY_BUFFER_STATE state;
};

static struct buffer_ctx *context_stack;
static struct input_file_ident input_file_ident;
static int sigchld_reset;
static struct sigaction oldsigchld;

static void
sigchld_default(void)
{
	if (!sigchld_reset) {
		struct sigaction act;
		act.sa_flags = 0;
		act.sa_handler = SIG_DFL;
		sigemptyset(&act.sa_mask);
		sigaction(SIGCHLD, &act, &oldsigchld);
		sigchld_reset = 1;
	}
}

static void
sigchld_restore(void)
{
	if (sigchld_reset) {
		sigaction(SIGCHLD, &oldsigchld, NULL);
		sigchld_reset = 0;
	}
}

#define STAT_ID_EQ(st,id) ((id).i_node == (st).st_ino       \
			    && (id).device == (st).st_dev)

static struct buffer_ctx *
ctx_lookup(struct stat *st)
{
	struct buffer_ctx *ctx;

	if (!context_stack)
		return NULL;

	for (ctx = context_stack; ctx; ctx = ctx->prev)
		if (STAT_ID_EQ(*st, ctx->id))
			break;
	return ctx;
}

static void
ctx_abort(void)
{
	/* Delete current input buffer */
	grecs_grecs__delete_buffer(YY_CURRENT_BUFFER);
			
	/* Clean-up context stack */
	while (context_stack) {
		struct buffer_ctx *prev = context_stack->prev;
		if (context_stack->id.pid > 0)
			kill(context_stack->id.pid, SIGKILL);
		fclose(context_stack->infile);
		grecs_free(context_stack);
		context_stack = prev;
	}
}

static struct grecs_symtab *incl_sources;

/* Calculate the hash of a struct input_file_ident.  */
static unsigned
incl_hasher(void *data, unsigned long n_buckets)
{
	const struct input_file_ident *id = data;
	return (id->i_node + id->device) % n_buckets;
}

/* Compare two input_file_idents for equality.  */
static int
incl_compare(void const *data1, void const *data2)
{
	const struct input_file_ident *id1 = data1;
	const struct input_file_ident *id2 = data2;
	return !(id1->device == id2->device && id1->i_node == id2->i_node);
}

static int
incl_copy(void *dst, void *src)
{
	memcpy(dst, src, sizeof(struct input_file_ident));
	return 0;
}

static int
source_lookup(struct stat *st)
{
	struct input_file_ident key;
	int install = 1;
  
	if (!incl_sources) {
		incl_sources = grecs_symtab_create(
			sizeof(struct input_file_ident), 
			incl_hasher,
			incl_compare,
			incl_copy,
			NULL,/*FIXME: alloc*/
			NULL);
		if (!incl_sources)
			grecs_alloc_die();
	}
	
	key.i_node = st->st_ino;
	key.device = st->st_dev;
	if (!grecs_symtab_lookup_or_install(incl_sources, &key, &install))
		grecs_alloc_die();
	return !install;
}

static int
push_source(const char *name, int once)
{
	FILE *fp;
	pid_t pid;
	struct buffer_ctx *ctx;
	struct stat st;
	int rc = stat(name, &st);

	if (context_stack) {
		if (rc) {
			grecs_error(&grecs_grecs_lloc, errno,
				    _("Cannot stat `%s'"), name);
			return 1;
		}

		if (grecs_current_locus_point.col &&
		    STAT_ID_EQ(st, input_file_ident)) {
			grecs_error(&grecs_grecs_lloc, 0, _("Recursive inclusion"));
			return 1;
		}
		
		if ((ctx = ctx_lookup(&st))) {
			grecs_error(&grecs_grecs_lloc, 0, _("Recursive inclusion"));
			if (ctx->prev)
				grecs_error(&ctx->prev->locus, 0,
					    _("`%s' already included here"),
					    name);
			else
				grecs_error(&grecs_grecs_lloc, 0,
					    _("`%s' already included at top level"),
					    name);
			return 1;
		}
	} else if (rc) {
		grecs_error(NULL, errno, _("Cannot stat `%s'"), name);
		return 1;
	}

	if (once && source_lookup(&st))
		return -1;

	if (grecs_preprocessor) {
		if (access(name,R_OK)) {
			grecs_error(context_stack ? &grecs_grecs_lloc : NULL, errno,
				    _("Cannot open `%s'"), name);
			return -1;
		}
		if (!context_stack) {
			/*
			 * Ensure default handling of SIGCHLD.
			 * It will be reset back to its original state
			 * by pop_source, when it has popped off the initial
			 * context.
			 */
			sigchld_default();
		}
		fp = grecs_preproc_extrn_start(name, &pid);
	} else {
		fp = fopen(name, "r");
		pid = -1;
	}
	if (!fp) {
		grecs_error(context_stack ? &grecs_grecs_lloc : NULL, errno,
			    _("Cannot open `%s'"), name);
		if (!context_stack)
			sigchld_restore();
		return 1;
	}

	/* Push current context */
	if (grecs_grecs_in) {
		ctx = grecs_malloc(sizeof(*ctx));
		ctx->locus = grecs_grecs_lloc;
		ctx->point = grecs_current_locus_point;
		ctx->id = input_file_ident;
		ctx->infile = grecs_grecs_in;
		ctx->state = YY_CURRENT_BUFFER;
		ctx->prev = context_stack;
		context_stack = ctx;
	}
	
	grecs_grecs_in = fp;
	grecs_grecs__switch_to_buffer(grecs_grecs__create_buffer(grecs_grecs_in, YY_BUF_SIZE));

	grecs_current_locus_point.file = grecs_install_text(name);
	grecs_current_locus_point.line = 1;
	grecs_current_locus_point.col = 0;
	
	input_file_ident.i_node = st.st_ino;
	input_file_ident.device = st.st_dev;
	input_file_ident.pid = pid;

	beginning_of_line = 1;
	
	if (grecs_grecs__flex_debug)
	  fprintf (stderr, "Processing file `%s'\n", name);

	if (preproc_mode)
		emit_syncline = 1;
	
	return 0;
}

static int
pop_source(void)
{
	struct buffer_ctx *ctx;

	fclose(grecs_grecs_in);
	if (input_file_ident.pid > 0) {
		if (grecs_preproc_extrn_shutdown(input_file_ident.pid)) {
			/* Abort further input */
			ctx_abort();

			/* Suppress further errors */
			grecs_grecs_grecs_error_suppress = 1;

			/* Indicate there is no more input */
			return 1;
		}
	}
	
	if (!context_stack) {
		if (grecs_grecs__flex_debug)
			fprintf(stderr, "End of input\n");
		/* Reset SIGCHLD to its original configuration */
		sigchld_restore();
		grecs_grecs_in = NULL;
		return 1;
	}
	
	/* Restore previous context */
	grecs_grecs_in = context_stack->infile;
	grecs_current_locus_point = context_stack->point;
	input_file_ident = context_stack->id;
	
	grecs_grecs__delete_buffer(YY_CURRENT_BUFFER);
	grecs_grecs__switch_to_buffer(context_stack->state);
	
	ctx = context_stack->prev;
	grecs_free(context_stack);
	context_stack = ctx;

	while (include_pos < include_glob.gl_pathc) {
		if (push_source(include_glob.gl_pathv[include_pos++],
				include_once) == 0)
			return 0;
	}

	if (include_glob.gl_pathc) {
		globfree(&include_glob);
		include_pos = include_glob.gl_pathc = 0;
	}
	
	if (grecs_grecs__flex_debug)
		fprintf(stderr, "Resuming file `%s' at line %lu\n",
			grecs_current_locus_point.file,
			(unsigned long) grecs_current_locus_point.line);

	if (preproc_mode) {
		input();
		grecs_locus_point_advance_line(grecs_current_locus_point);
		emit_syncline = 1;
	}
	
	return 0;
}

/*
 * Internal lexer functions.
 */
int
grecs_grecs_wrap()
{
	return pop_source();
}

int
grecs_lex_begin(const char *name, int trace)
{
	grecs_grecs__flex_debug = trace;
	grecs_line_acc_create();
	return push_source(name, 1);
}

void
grecs_lex_end(int err)
{
	grecs_symtab_clear(incl_sources);
	grecs_line_acc_free();
}

/*
 * Here-document functions.
 */
static int
isemptystr(int off)
{
	for (; grecs_grecs_text[off] && isspace(grecs_grecs_text[off]); off++)
		;
	return grecs_grecs_text[off] == ';' || grecs_grecs_text[off] == 0;
}

char *
multiline_strip_tabs(char *text)
{
	if (char_to_strip)
		for (; *text && char_to_strip(*text); text++)
			;
	return text;
}

static void
multiline_add(char *s)
{
	if (multiline_unescape) {
		for (; *s; s++)	{
			if (*s == '\\') {
				grecs_line_acc_grow_char_unescape(s[1]);
				++s;
			} else
				grecs_line_acc_grow_char(*s);
		}
	} else
		grecs_line_add(s, strlen(s));
}

static int
is_tab(char c)
{
	return c == '\t';
}
 
static int
is_ws(char c)
{
	return c == '\t' || c == ' ';
}

void
multiline_begin(char *p)
{
	if (*p == '-') {
		if (*++p == ' ') {
			char_to_strip = is_ws;
			p++;
		} else
			char_to_strip = is_tab;
	} else
		char_to_strip = NULL;
	if (*p == '\\') {
		p++;
		multiline_unescape = 0;
		multiline_delimiter_len = strcspn(p, " \t");
	} else if (*p == '"') {
		char *q;
 
		p++;
		multiline_unescape = 0;
		q = strchr(p, '"');
		multiline_delimiter_len = q - p;
	} else {
		multiline_delimiter_len = strcspn(p, " \t");
		multiline_unescape = 1;
	}

	/* Remove trailing newline */
	multiline_delimiter_len--;
	multiline_delimiter = grecs_malloc(multiline_delimiter_len + 1);
	memcpy(multiline_delimiter, p, multiline_delimiter_len);
	multiline_delimiter[multiline_delimiter_len] = 0;
	grecs_line_begin();

	/* Update locus */
	grecs_locus_point_advance_line(grecs_current_locus_point);
	if (grecs_parser_options & GRECS_OPTION_ADJUST_STRING_LOCATIONS) {
		grecs_grecs_lloc.beg = grecs_current_locus_point;
		grecs_grecs_lloc.beg.col++;
	}
}

/*
 * Auxiliary lexer functions.
 */
static int
string_or_ident(void)
{
	char *p = grecs_grecs_lval.string;
	if (isalpha(*p) || *p == '_') {
		while (*++p) {
			if (!(isalnum(*p) || *p == '_' || *p == '-'))
				return STRING;
		}
		return IDENT;
	}
	return STRING;
}

static void
qstring_locus_fixup(void)
{
	if (grecs_parser_options & GRECS_OPTION_ADJUST_STRING_LOCATIONS) {
		grecs_grecs_lloc.beg.col++;
		grecs_grecs_lloc.end.col--;
	}
}

grecs_value_t *
grecs_value_ptr_from_static(grecs_value_t *input)
{
	grecs_value_t *ptr = grecs_malloc(sizeof(*ptr));
	*ptr = *input;
	return ptr;
}

/*
 * Syncline directive parsers.
 */

static int
assign_locus(struct grecs_locus_point *ploc, char *name, char *line)
{
	char *p;
	
	if (name) {
		ploc->file = grecs_install_text(name);
	}
	ploc->line = strtoul(line, &p, 10);
	ploc->col = 0;
	return *p != 0;
}

void
grecs_parse_line_directive(char *text, grecs_locus_t *ploc,
			   struct grecs_locus_point *ppoint)
{
	int rc = 1;
	struct wordsplit ws;
	
	if (wordsplit(text, &ws, WRDSF_DEFFLAGS))
		grecs_error(ploc, 0, _("cannot parse #line line: %s"),
			    wordsplit_strerror(&ws));
	else if (ws.ws_wordc == 2)
		rc = assign_locus(ppoint, NULL, ws.ws_wordv[1]);
	else if (ws.ws_wordc >= 3) 
		rc = assign_locus(ppoint, ws.ws_wordv[2], ws.ws_wordv[1]);
	else 
		grecs_error(ploc, 0, _("invalid #line statement"));
	
	if (rc) 
		grecs_error(ploc, 0, _("malformed #line statement"));
	wordsplit_free(&ws);
}

void
grecs_parse_line_directive_cpp(char *text, grecs_locus_t *ploc,
			       struct grecs_locus_point *ppoint)
{
	struct wordsplit ws;
	
	if (wordsplit(text, &ws, WRDSF_DEFFLAGS))
		grecs_error(ploc, 0, _("cannot parse #line line: %s"),
			    wordsplit_strerror(&ws));
	else if (ws.ws_wordc < 3)
		grecs_error(ploc, 0, _("invalid #line statement"));
	else if (assign_locus(ppoint, ws.ws_wordv[2], ws.ws_wordv[1]))
		grecs_error(ploc, 0, _("malformed #line statement"));
	wordsplit_free(&ws);
}

/*
 * External preprocessor support.
 */
const char *grecs_preprocessor = NULL; /* Preprocessor command */
int grecs_log_to_stderr = 1;           /* Log to stderr or to syslog? */  
void (*grecs_log_setup_hook) () = NULL;/* Log subsystem setup routine. */

static int
preproc_file(const char *file)
{
	size_t size = 0;
	char *cmd = NULL;
	char *setup_file;
	char *argv[4];
	int rc;
	
	setup_file = grecs_find_include_file("pp-setup", 0);
	if (setup_file)
		rc = grecs_asprintf(&cmd, &size,
				    "%s \"%s\" \"%s\"",
				    grecs_preprocessor, setup_file, file);
	else
		rc = grecs_asprintf(&cmd, &size,
				    "%s \"%s\"", grecs_preprocessor, file);
	if (rc)
		grecs_alloc_die();
	if ((argv[0] = getenv("SHELL")) == NULL)
		argv[0] = "/bin/sh";
	argv[1] = "-c";
	argv[2] = cmd;
	argv[3] = NULL;
	execvp(argv[0], argv);
	return -1;
}

FILE *
grecs_preproc_extrn_start(const char *file_name, pid_t *ppid)
{
	int pout[2];
	pid_t pid;
	int i;
	FILE *fp = NULL;
	int rc;

	fflush(grecs_grecs_out);
	if (pipe(pout)) {
		grecs_error(NULL, errno, "pipe");
		return NULL;
	}
	switch (pid = fork()) {
		/* The child branch.  */
	case 0:
		if (pout[1] != 1) {
			if (dup2(pout[1], 1) == -1) {
				grecs_error(NULL, errno, "dup2");
				exit(127);
			}
		}

		if (!grecs_log_to_stderr) {
			int p[2];
			char *buf = NULL;
			size_t size = 0;
			FILE *fp;
			
			signal(SIGCHLD, SIG_DFL);
			if (pipe(p)) {
				grecs_error(NULL, errno, "pipe");
				exit(127);
			}
			switch (pid = fork()) {
				/* Grandchild */
			case 0:
				if (p[1] != 2 && dup2(p[1], 2) == -1) {
					grecs_error(NULL, errno, "dup2");
					exit(127);
				}
				close(p[0]);

				rc = preproc_file(file_name);
				break;
				
			case -1:
				/*  Fork failed */
				if (grecs_log_setup_hook)
					grecs_log_setup_hook();
				grecs_error(NULL, errno, _("Cannot run `%s'"),
					    grecs_preprocessor);
				exit(127);
				
			default:
				/* Sub-master */
				close (p[1]);
				fp = fdopen(p[0], "r");
				if (grecs_log_setup_hook)
					grecs_log_setup_hook();
				while (grecs_getline(&buf, &size, fp) > 0) {
					size_t n = strlen(buf);
					if (buf[n-1] == '\n')
						buf[n-1] = 0;
					grecs_error(NULL, 0, "%s", buf);
				}
				wait(&i);
				if (WIFEXITED(i))
					rc = WEXITSTATUS(i);
				else
					rc = 127;
			}
		} else {
			rc = preproc_file(file_name);
		}
		if (rc == -1)
			exit(127);
		exit(rc);

	case -1:
		/*  Fork failed */
		grecs_error(NULL, errno, _("Cannot run `%s'"),
			    grecs_preprocessor);
		break;
		
	default:
		close(pout[1]);
		fp = fdopen(pout[0], "r");
		break;
	}
	*ppid = pid;
	return fp;
}

int
grecs_preproc_extrn_shutdown(pid_t pid)
{
	int status;

	if (waitpid(pid, &status, 0) != pid) {
	        grecs_error(NULL, errno, "waitpid(%d)", pid);
	} else if (WIFEXITED(status)) {
		if (WEXITSTATUS(status)) {
			grecs_error(NULL, 0, _("%s exited with status %d"),
				    grecs_preprocessor, WEXITSTATUS(status));
			return -1;
		}
	} else if (WIFSIGNALED(status)) {
		grecs_error(NULL, 0, _("%s terminated on signal %d"),
			    grecs_preprocessor, WTERMSIG(status));
		return -1;
	} else {
		grecs_error(NULL, 0, _("%s terminated abnormally (%d)"),
			    grecs_preprocessor, status);
		return -1;
	}
	return 0;
}

int
grecs_preprocess(char const *name, int trace)
{
	int rc;
	
	grecs_grecs__flex_debug = trace;
	grecs_grecs_out = stdout;
	grecs_line_acc_create();
	preproc_mode = 1;
	rc = push_source(name, 1);
	if (rc == 0) {
		int t;
		while ((t = grecs_grecs_lex()) != 0) {
			if (trace) {
				char *p;

				YY_LOCATION_PRINT(stderr, grecs_grecs_lloc);
				fprintf(stderr, ": %d: ", t);
				switch (t) {
				case STRING:
				case QSTRING:
				case MSTRING:
					fputc('"', stderr);
					for (p = grecs_grecs_lval.string; *p; p++) {
						if (*p == '\\' || *p == '"')
							fputc('\\', stderr);
						fputc(*p, stderr);
					}
					fputc('"', stderr);
					break;
					
				case IDENT:
					fprintf(stderr, "%s", grecs_grecs_lval.string);
					break;
					
				default:
					fputc(t, stderr);
				}
				fputc('\n', stderr);
			}
		}
		grecs_lex_end(0);
	}
	return rc;
}

/*
 * Additional function.
 *
 * Not used by the lexer, but retained for backward compatibility.
 */
ssize_t
grecs_getline(char **pbuf, size_t *psize, FILE *fp)
{
	char *buf = *pbuf;
	size_t size = *psize;
	ssize_t off = 0;
  
	if (!buf) {
		size = 1;
		buf = grecs_malloc(size);
	}
	
	do {
		if (off == size - 1) {
			size_t nsize = 2 * size;
			if (nsize < size)
				grecs_alloc_die();
			buf = grecs_realloc(buf, nsize);
			size = nsize;
		}
		if (!fgets(buf + off, size - off, fp)) {
			if (off == 0)
				off = -1;
			break;
		}
		off += strlen(buf + off);
	} while (buf[off - 1] != '\n');

	*pbuf = buf;
	*psize = size;
	return off;
}
					

	

